/*
 * @(#)file      RMIConnector.java
 * @(#)author    Mats Mortensen
 * @(#)version   1.1
 * @(#)lastedit  18.4.2005
 *
 */
package coms;

import java.io.IOException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

public class RMIConnector {
	private JMXServiceURL url;
	private JMXConnector jmxc;
	private MBeanServerConnection mbsc;
	//private String domain;

	/**
	 * Attempts to connect to the CoMS using RMI
	 * 
	 * @param String
	 *            host, String port, String bindName
	 */
	public void connect(String host, String port, String bindName)
			throws IOException, SecurityException {
		url = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://" + host + ":"
				+ port + "/" + bindName);
		jmxc = JMXConnectorFactory.connect(url, null);
		mbsc = jmxc.getMBeanServerConnection();
	}

	/**
	 * Attempts to close connection to the CoMS
	 */
	public void close() {
		try {
			if (jmxc != null)
				jmxc.close();
		}
		catch (IOException e) {}
	}

	/**
	 * Invokes a function in a output component
	 * 
	 * @param ObjectName
	 *            outputComponent, String method, Object[] param, String[] types
	 */
	public Object invoke(ObjectName outputComponent, String method,
			Object[] param, String[] types) {
		try {
			return mbsc.invoke(outputComponent, method, param, types);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Gets an attribute (object) defined in a output component
	 * 
	 * @param ObjectName
	 *            outputComponent, String attr
	 */
	public Object getAttribute(ObjectName outputComponent, String attr) {
		try {
			return mbsc.getAttribute(outputComponent, attr);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}
