package coms.component;

import java.util.ArrayList;
import java.util.List;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.NotificationEmitter;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;

public abstract class AbstractNotifier extends Component implements
		NotificationEmitter {
	protected boolean verbose;
	protected List<NotificationListener> listeners;
	protected List<NotificationFilter> filters;
	protected List<Object> handbacks;
	protected long sequence;

	public AbstractNotifier() {
		super();
		listeners = new ArrayList<NotificationListener>();
		filters = new ArrayList<NotificationFilter>();
		handbacks = new ArrayList<Object>();
		sequence = 0;
		verbose = false;
	}

	public void addNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback) {
		listeners.add(listener);
		filters.add(filter);
		handbacks.add(handback);
	}

	public void removeNotificationListener(NotificationListener listener) {
		for (int i = 0; i < listeners.size(); i++) {
			if (listeners.equals(listener)) {
				listeners.remove(i);
				filters.remove(i);
				handbacks.remove(i);
			}
		}
	}

	public void removeNotificationListener(NotificationListener listener,
			NotificationFilter filter, Object handback) {
		for (int i = 0; i < listeners.size(); i++) {
			if (listeners.equals(listener)) {
				listeners.remove(i);
				filters.remove(i);
				handbacks.remove(i);
			}
		}
	}

	protected void debug(String msg) {
		for (int z = 0; z < listeners.size(); z++) {
			Notification not = new Notification("Debug Info", this, sequence,
					msg);
			if (filters.get(z) == null
					|| filters.get(z).isNotificationEnabled(not)) {
				listeners.get(z).handleNotification(not, handbacks.get(z));
			}
		}
		sequence++;
		if(verbose) {
			System.out.println(msg);
		}
	}

	protected void send(Notification not) {
		for (int z = 0; z < listeners.size(); z++) {
			if (filters.get(z) == null
					|| filters.get(z).isNotificationEnabled(not)) {
				listeners.get(z).handleNotification(not, handbacks.get(z));
			}
		}
		sequence++;
	}

	protected void error(String msg) {
		for (int z = 0; z < listeners.size(); z++) {
			Notification not = new Notification("Error", this, sequence, msg);
			if (filters.get(z) == null
					|| filters.get(z).isNotificationEnabled(not)) {
				listeners.get(z).handleNotification(not, handbacks.get(z));
			}
		}
		sequence++;
		if(verbose) {
			System.out.println(msg);
		}
	}

	public MBeanNotificationInfo[] getNotificationInfo() {
		MBeanNotificationInfo[] not = new MBeanNotificationInfo[2];
		not[0] = new MBeanNotificationInfo(new String[] { "Debug Info" },
				"Debug Info", "Reports all debug info.");
		not[1] = new MBeanNotificationInfo(new String[] { "Error" },
				"Error messages", "Reports errors");
		return not;
	}

	public boolean isVerbose() {
		return verbose;
	}

	public void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}
}
