package coms.component;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

public abstract class Binding {
	/**
	 * Reads "numBytes" from a sensor and returns them casted as Object.
	 * Disconnection is done when reception is completed.
	 */
	public abstract DataInputStream read() throws ClassNotFoundException,
			ConnectException, IOException, SocketTimeoutException;

	/**
	 * Sends "message" to a sensor. Disconnection is done when sending is
	 * completed.
	 */
	public abstract void write(Object obj) throws ConnectException,
			UnknownHostException, IOException, SocketTimeoutException;

	public abstract void write(String msg) throws IOException,
			UnknownHostException;

	public abstract void connect(String host, int port, int timeout);

	/**
	 * Returns a byte array from the given object.
	 * 
	 * @param object
	 *            to convert
	 * @return byte array from the object
	 */
	protected byte[] objectToBytes(Object object) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream os = new ObjectOutputStream(baos);
		os.writeObject(object);
		os.close();
		return baos.toByteArray();
	}

	/**
	 * Returns a object from the given byte array.
	 * 
	 * @param bytes
	 *            array to convert
	 * @return object
	 */
	protected Object bytesToObject(byte[] bytes) throws IOException,
			ClassNotFoundException {
		ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
		ObjectInputStream is = new ObjectInputStream(bais);
		return is.readObject();
	}
}
