/*
 * @(#)file      ConfigComponent.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  29.6.2005
 * 
 */
package coms.component;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.ConnectException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.HashMap;

/**
 * Class which provides core functionality for config components
 */
public abstract class ConfigComponent extends AbstractNotifier {
	private Binding binding;

	public ConfigComponent() {
		super();
	}

	/**
	 * Initializes all config components
	 */
	public final void doInit() {
		// Create a new binding for the component
		this.binding = locateBinding(metadata.getBindingType(), metadata
				.getCustomOptions());
	}

	/**
	 * Creates a new binding object to be used depending on the binding type the
	 * component has specified
	 * 
	 * @param String
	 *            type, HashMap param
	 */
	private Binding locateBinding(String type, HashMap<String, String> param) {
		Binding binding = null;
		if (type.equals("TCP"))
			binding = new TCPBinding(param.get("host"), new Integer(param
					.get("timeout")).intValue(), new Integer(param.get("port"))
					.intValue());
		else if (type.equals("UDP"))
			binding = new UDPBinding();
		return binding;
	}

	/**
	 * Reads from sensor
	 * 
	 * @param int
	 *            bytes
	 * @return Object
	 */
	public DataInputStream read() throws ClassNotFoundException,
			ConnectException, IOException, SocketTimeoutException {
		return binding.read();
	}

	/**
	 * Writes to sensor
	 * 
	 * @param Object
	 *            obj
	 */
	public void write(Object obj) throws ConnectException,
			UnknownHostException, IOException, SocketTimeoutException {
		binding.write(obj);
	}

	/**
	 * Method not used by config components
	 */
	public final void execute() {
		Thread.yield();
	}

	/**
	 * Method not used by config components
	 */
	public final void onEvent(String table) {
		Thread.yield();
	}
}
