/*
 * @(#)file      OuputComponent.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  29.6.2005
 * 
 */
package coms.component;

import java.util.HashSet;
import java.util.Iterator;
import javax.management.ObjectName;

/**
 * Class which provides core functionality for output components
 */
public abstract class OutputComponent extends AbstractNotifier {
	public OutputComponent() {
		super();
	}

	/**
	 * Initializes all output components
	 */
	public final void doInit() {
		HashSet<String> set = metadata.getSubscriptions();
		Iterator it = set.iterator();
		// Subscribe for events
		while (it.hasNext())
			subscribe((String) it.next());
		// Register as a xmlrpc handler
		createXmlRpcBinding();
	}

	/**
	 * Registers an XML-RPC binding
	 */
	private void createXmlRpcBinding() {
		try {
			ObjectName handler = createObjectName("XmlRpcHandler", "Core");
			server.invoke(handler, "addHandler", new Object[] {
					metadata.getName(), this }, new String[] {
					String.class.getName(), Object.class.getName() });
		}
		catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
	}

	/**
	 * Method not used by output components
	 */
	public final void execute() {
		Thread.yield();
	}
}
