/*
 * @(#)file      ProcessingComponent.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  29.6.2005
 * 
 */
package coms.component;

import javax.management.MBeanException;
import javax.management.ObjectName;
import coms.core.inference.EngineNotFoundException;
import coms.tools.inference.Engine;

/**
 * Class which provides core functionality for processing components
 */
public abstract class ProcessingComponent extends AbstractNotifier {
	public ProcessingComponent() {
		super();
	}

	/**
	 * Initialize all processing components
	 */
	public final void doInit() {}

	/**
	 * This method is not used by processing components
	 */
	public final void onEvent(String table) {
		Thread.yield();
	}

	public final Engine createEngine(Boolean isPersistent) {
		try {
			ObjectName factory = createObjectName("InferenceFactory", "Core");
			return (Engine) server.invoke(factory, "createEngine",
					new Object[] { isPersistent, componentID }, new String[] {
							Boolean.class.getName(), String.class.getName() });
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public final Engine getEngine(String componentID)
			throws EngineNotFoundException {
		try {
			ObjectName factory = createObjectName("InferenceFactory", "Core");
			return (Engine) server.invoke(factory, "getEngine",
					new Object[] { componentID }, new String[] { String.class
							.getName() });
		}
		catch (MBeanException e) {
			throw new EngineNotFoundException(e.getCause().getMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
