package coms.component;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;

public class TCPBinding extends Binding {
	private static final int __TimeMultiplicator = 1000; // 1s == 1000ms!
	private Socket socket = null;
	private InputStream input = null;
	private OutputStream output = null;
	private String host;
	private int port;
	private int timeout;

	public TCPBinding(String host, int port, int timeout) {
		this.host = host;
		this.port = port;
		this.timeout = timeout;
	}

	public void connect(String host, int port, int timeout) {
		this.host = host;
		this.port = port;
		this.timeout = timeout;
	}

	/**
	 * Connect to a server and set "input" and "output" for communication. Give
	 * up after "timeOut" second if connection can not be done.
	 */
	private void doConnect(String server, int portNum, int timeOut)
			throws ConnectException, UnknownHostException, IOException {
		InetAddress inetAddr = InetAddress.getByName(server);
		SocketAddress sockAddr = new InetSocketAddress(inetAddr, portNum);
		socket = new Socket();
		socket.connect(sockAddr, timeOut * __TimeMultiplicator);
		input = socket.getInputStream();
		output = socket.getOutputStream();
		return;
	}

	/**
	 * Connect to a server and set "input" and "output" for communication. Block
	 * until connection is done.
	 */
	private void doConnect(String server, int portNum) throws ConnectException,
			UnknownHostException, IOException {
		// InetAddress inetAddr = InetAddress.getByName(server);
		socket = new Socket(server, portNum); // Blocks until connection
		// succeeds
		input = socket.getInputStream();
		output = socket.getOutputStream();
		return;
	}

	/**
	 * End connection and clean various stuff.
	 */
	private void disconnect() throws IOException {
		input.close();
		output.close();
		socket.close();
		return;
	}

	/**
	 * Reads "numBytes" from a sensor and returns them casted as Object.
	 * Connection is done via TCP. Disconnection is done when reception is
	 * completed.
	 */
	public synchronized DataInputStream read() throws ConnectException,
			IOException, SocketTimeoutException, ClassNotFoundException {
		if (timeout == 0)
			doConnect(host, port);
		else
			doConnect(host, port, timeout);
		int size = input.available();
		;
		byte[] inputBuf;
		while (size == 0) {
			size = input.available();
			Thread.yield();
		}
		inputBuf = new byte[size];
		input.read(inputBuf, 0, size);
		disconnect();
		ByteArrayInputStream bstream = new ByteArrayInputStream(inputBuf);
		return new DataInputStream(bstream);
	}

	/**
	 * Sends "message" to a sensor. Connection is done via TCP. Disconnection is
	 * done when sending is completed.
	 */
	public synchronized void write(Object message) throws ConnectException,
			UnknownHostException, IOException, SocketTimeoutException {
		if (timeout == 0)
			doConnect(host, port);
		else
			doConnect(host, port, timeout);
		output.write(objectToBytes(message));
		output.flush();
		disconnect();
		return;
	}

	public void write(String msg) throws IOException, UnknownHostException {
		if (timeout == 0)
			doConnect(host, port);
		else
			doConnect(host, port, timeout);
		output.write(msg.getBytes());
		output.flush();
		disconnect();
		return;
	}
}
