package coms.config;

import java.util.HashMap;
import java.util.HashSet;
import coms.metadata.Table;

/**
 * This is a class that provides the configuration options
 * 
 * @author Mats Mortensen
 */
public class ConfigOptions {
	private String dbtype;
	private String dbname;
	private String dbuser;
	private String dbpass;
	private String dbhome;
	private HashSet<Table> tables;
	private HashMap<String, String> parameters;

	public ConfigOptions() {}

	/**
	 * Sets the system database type
	 */
	public void setDBType(String type) {
		this.dbtype = type;
	}

	/**
	 * Sets the system database name
	 */
	public void setDBName(String name) {
		this.dbname = name;
	}

	/**
	 * Sets the system database user
	 */
	public void setDBUser(String user) {
		this.dbuser = user;
	}

	/**
	 * Sets the system database password
	 */
	public void setDBPass(String pass) {
		this.dbpass = pass;
	}

	/**
	 * Sets the system database home location
	 */
	public void setDBHome(String home) {
		this.dbhome = home;
	}

	/**
	 * Sets the system parameters
	 */
	public void setParameters(HashMap<String, String> map) {
		this.parameters = map;
	}

	/**
	 * Sets the system tables
	 */
	public void setSystemTables(HashSet<Table> set) {
		this.tables = set;
	}

	/**
	 * Returns the system parameters
	 */
	public HashMap<String, String> getParameters() {
		return this.parameters;
	}

	/**
	 * Returns the system database tables
	 */
	public HashSet<Table> getSystemTables() {
		return this.tables;
	}

	/**
	 * Returns the system database type
	 */
	public String getDBType() {
		return this.dbtype;
	}

	/**
	 * Returns the system database name
	 */
	public String getDBName() {
		return this.dbname;
	}

	/**
	 * Returns the system database username
	 */
	public String getDBuser() {
		return this.dbuser;
	}

	/**
	 * Returns the system database password
	 */
	public String getDBPass() {
		return this.dbpass;
	}

	/**
	 * Returns the system database home location
	 */
	public String getDBHome() {
		return this.dbhome;
	}

	/**
	 * Returns the moduledomain
	 * 
	 * @return moduleDomain
	 */
	public String getModuleDomain() {
		return parameters.get("ModuleDomain").trim();
	}

	/**
	 * Returns component domain
	 * 
	 * @return componenDomain
	 */
	public String getComponentDomain() {
		return parameters.get("ComponentDomain").trim();
	}

	/**
	 * Returns module type
	 * 
	 * @return moduletype
	 */
	public String getModuleType() {
		return parameters.get("ModuleType").trim();
	}

	/**
	 * Returns moduleagent class
	 * 
	 * @return moduleagentclass
	 */
	public String getModuleAgentClass() {
		return parameters.get("ModuleAgentClass").trim();
	}

	/**
	 * Returns MLet Path
	 * 
	 * @return MLetPath
	 */
	public String getMLetPath() {
		return parameters.get("MLetPath").trim();
	}

	/**
	 * Returns URL
	 * 
	 * @return listenURL
	 */
	public String getModuleListenURL() {
		String url = parameters.get("ListenURL").trim() + ":";
		url += parameters.get("ListenPort").trim() + "/"
				+ parameters.get("ModuleBindingName").trim();
		return url;
	}

	/**
	 * Returns URL
	 * 
	 * @return listenURL
	 */
	public String getComponentListenURL() {
		String url = parameters.get("ListenURL").trim() + ":";
		url += parameters.get("ListenPort").trim() + "/"
				+ parameters.get("ComponentBindingName").trim();
		return url;
	}

	/**
	 * Returns RMI shell command
	 * 
	 * @return RMIShellCommand
	 */
	public String getRMIShellCommand() {
		return parameters.get("RMIShellCommand").trim();
	}

	/**
	 * Returns RMI shell argument
	 * 
	 * @return rmiShellArg
	 */
	public String getRMIShellArg() {
		return parameters.get("RMIShellArg").trim();
	}

	/**
	 * Returns logger name
	 * 
	 * @return loggerName
	 */
	public String getLoggerName() {
		return parameters.get("LoggerName").trim();
	}

	/**
	 * Returns logger class
	 * 
	 * @return loggerClass
	 */
	public String getLoggerClass() {
		return parameters.get("LoggerClass").trim();
	}

	/**
	 * Returns logger status
	 * 
	 * @return loggerStatus
	 */
	public boolean getLoggerStatus() {
		if (parameters.get("LoggerStatus").trim().toLowerCase().equals("true"))
			return true;
		else
			return false;
	}

	/**
	 * Returns print to file
	 * 
	 * @return printToFile
	 */
	public boolean getLoggerPrintToFile() {
		if (parameters.get("LoggerPrintToFile").trim().toLowerCase().equals(
				"true"))
			return true;
		else
			return false;
	}

	/**
	 * Returns print to screen
	 * 
	 * @return printToScreen
	 */
	public boolean getLoggerPrintToScreen() {
		if (parameters.get("LoggerPrintToScreen").trim().toLowerCase().equals(
				"true"))
			return true;
		else
			return false;
	}

	/**
	 * Returns boolean clear log at start
	 * 
	 * @return true or false
	 */
	public boolean getClrLogAtStart() {
		if (parameters.get("ClrLogAtStart").trim().toLowerCase().equals("true"))
			return true;
		else
			return false;
	}

	/**
	 * Returns log file
	 * 
	 * @return logFile
	 */
	public String getLogFile() {
		return parameters.get("LogFile").trim();
	}

	/**
	 * Returns deployment directory
	 * 
	 * @return deployDir
	 */
	public String getDeployDir() {
		return parameters.get("DeployDir").trim();
	}

	/**
	 * Returns authentication password
	 * 
	 * @return authPass
	 */
	public String getAuthPass() {
		return parameters.get("AuthPass").trim();
	}
}
