package coms.config;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import coms.metadata.Table;
import coms.metadata.TableField;

public class ConfigParser implements ContentHandler {
	
	private XMLReader 							parser;
	private ConfigOptions 						config;
	private HashMap<String, String> 			parameters;
	private String 								fileurl;
	@SuppressWarnings("unused")
	private String 								starttag, endtag, fullvalue="";
	private String								state1 = "";
	private String								state2 = "";
	private String								state3 = "";
	
	private HashSet<Table> 						systemtables;
	private Table 								table;
	private TableField 							field;
	private String 								primary = "";
	
	
	public void setDocumentLocator(Locator locator) {
		
	}
	
	public void startDocument() throws SAXException {
		
	}
	
	public void endDocument() throws SAXException {
		
	}
	
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
	throws SAXException {
		
		starttag = localName;
		
		// First level tags/state
		if(starttag.equals("SYSTEM-DB")){
			state1 = "SYSTEM-DB";

		}else if(starttag.equals("SYSTEM-TABLES")){
			state1 = "SYSTEM-TABLES";
			systemtables = new HashSet<Table>();
		}else if(starttag.equals("SYSTEM-PARAMETERS")){
			state1 = "SYSTEM-PARAMETERS";
			parameters = new HashMap<String, String>();
		}
			
		// Second level tags/state
		if(starttag.equals("table")){
			state2 = "table";
			table = new Table();
		}
		
		// Third level tags/state
		if(starttag.equals("field")){
			state3 = "field";
			field = new TableField();
		}
		

			
		
	}
	
	public void endElement(String namespaceURI, String localName, String qName)
	throws SAXException {
		
		endtag = localName;
		
		if(state1.equals("SYSTEM-DB")){
			if(endtag.equals("type"))
				config.setDBType(fullvalue);
			else if(endtag.equals("name"))
				config.setDBName(fullvalue);
			else if(endtag.equals("user"))
				config.setDBUser(fullvalue);
			else if(endtag.equals("pass"))
				config.setDBPass(fullvalue);
			else if(endtag.equals("home"))
				config.setDBHome(fullvalue);
				
		}else if(state1.equals("SYSTEM-TABLES")){
			if(state2.equals("table")){
				
				if(endtag.equals("tablename"))
					table.setName(fullvalue);
				else if(endtag.equals("table"))
					systemtables.add(table);
				else if(endtag.equals("primarykey"))
					primary = fullvalue;
				
				if(state3.equals("field")){
					
					if(endtag.equals("fieldname"))
						field.setName(fullvalue);
					else if(endtag.equals("fieldtype"))
						field.setType(fullvalue);
					else if(endtag.equals("field")){
						if(field.getName().equalsIgnoreCase(primary))
							table.addPrimaryKey(field.getName(), field.getType());
						else
							table.addField(field);
					}else if(endtag.equals("default"))
						field.setDefault(fullvalue);
					
				}
			}
			
		}else if(state1.equals("SYSTEM-PARAMETERS")){
			if(!endtag.equals(""))	
				parameters.put(starttag, fullvalue);
		}
	
		
		starttag = "";
		endtag = "";
		fullvalue = "";	 
	}
	
	
	public void characters(char[] text, int start, int length)throws SAXException {
		String value = new String(text, start, length);		
		value = value.trim();
		fullvalue += value;
		
	}
	
	public void ignorableWhitespace(char[] text, int start, int length)
	throws SAXException {
		
	}
	
	public void processingInstruction(String target, String data)
	throws SAXException {
		
	}
	
	public void startPrefixMapping(String prefix, String uri)
	throws SAXException {
		
	}
	
	public void endPrefixMapping(String prefix) throws SAXException {
		
	}
	
	public void skippedEntity(String name) throws SAXException {
		
	}
	
	
	public ConfigParser(String configfile) throws SAXException{
		try {
			// XMLReaderFactory factory;
			parser = XMLReaderFactory.createXMLReader();
			parser.setFeature("http://xml.org/sax/features/validation", false);
			fileurl = configfile;
			
		}catch (SAXNotRecognizedException ex){
			System.out.println("Installed XML parser cannot validate");
			System.exit(0);
		}catch (SAXNotSupportedException e){
			System.out.println("Installed XML parser cannot validate");
			System.exit(0);
		}
		
		// Install the content handler
		parser.setContentHandler(this);
		
	}
	
	public ConfigOptions parse() throws SAXException, SAXParseException{
		try {
			config = new ConfigOptions();
			parser.parse(fileurl);
			config.setParameters(parameters);
			config.setSystemTables(systemtables);
			
			return config;
		}catch (IOException e) {
			System.out.println("Error reading the Configuration from: "+fileurl + " when parsing. \nCaused by: "+e.getMessage());
		}
		
		return null;	
		
	}
}