package coms.core;

import java.io.IOException;
import java.util.HashMap;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Class for getting a xml file into a hashmap
 * 
 * @author Tom Arild Jakobsen & Mats Mortensen
 */
public class Config implements ContentHandler {
	String starttag, endtag;
	HashMap<String, String> config = new HashMap<String, String>();

	/**
	 * Class Constructor
	 */
	public Config() {}

	/**
	 * Function for reading a xml file containing the configuration to a hashmap
	 * 
	 * @param fileurl
	 *            Path of the xml file.
	 * @return Hashmap with the tags as keys and the contentes between the tags
	 *         as values.
	 */
	public static HashMap<String, String> read(String fileurl)
			throws IOException {
		XMLReader parser;
		Config con = new Config();
		try {
			parser = XMLReaderFactory.createXMLReader();
		}
		catch (Exception e) {
			// fall back on Xerces parser by name
			try {
				parser = XMLReaderFactory
						.createXMLReader("org.apache.xerces.parsers.SAXParser");
			}
			catch (Exception ee) {
				System.err.println("Couldn't locate a SAX parser");
				return null;
			}
		}
		// Install the content handler
		parser.setContentHandler(con);
		// command line should offer URIs or file names
		try {
			parser.parse(fileurl);
		}
		catch (SAXParseException e) {
			throw new IOException("File \"" + fileurl
					+ "\" is not well formed at line " + e.getLineNumber());
		}
		catch (SAXException e) {
			throw new IOException(e.getMessage());
		}
		catch (IOException e) {
			throw new IOException("Can't find file \"" + fileurl + "\"");
		}
		return con.config;
	}

	/**
	 * Action to be taken when parsing a starttag.
	 */
	public void startElement(String namespaceURI, String localName,
			String qName, Attributes atts) throws SAXException {
		starttag = localName;
	}

	/**
	 * Action to be taken when parsing an endtag.
	 */
	public void endElement(String namespaceURI, String localName, String qName)
			throws SAXException {
		endtag = localName;
		starttag = "";
		endtag = "";
	}

	/**
	 * Action to be takel when parsing something else then a start or end tag.
	 */
	public void characters(char[] text, int start, int length)
			throws SAXException {
		if (!starttag.equals("") && !starttag.equals("config")) {
			config.put(starttag, new String(text, start, length).trim());
		}
	}

	public void setDocumentLocator(Locator locator) {}

	public void startDocument() throws SAXException {}

	public void endDocument() throws SAXException {}

	public void ignorableWhitespace(char[] text, int start, int length)
			throws SAXException {}

	public void processingInstruction(String target, String data)
			throws SAXException {}

	public void startPrefixMapping(String arg0, String arg1)
			throws SAXException {}

	public void endPrefixMapping(String arg0) throws SAXException {}

	public void skippedEntity(String arg0) throws SAXException {}
}
