package coms.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import coms.metadata.MetaData;

/**
 * @author Dan Peder Eriksen
 */
public class Graph {
	private HashMap<String, MetaData> map = new HashMap<String, MetaData>();
	private HashSet<String> cycleSet = new HashSet<String>();

	public Graph() {
		super();
	}

	public void addNode(MetaData node) {
		map.put(node.getName(), node);
	}

	public MetaData getNode(String name) {
		return (MetaData) map.get(name);
	}

	public void removeNode(MetaData com) {
		List list = allWhoIDependOn(com);
		Iterator it = list.iterator();
		while (it.hasNext()) {
			MetaData depOn = (MetaData) it.next();
			depOn.removeDependsOnMe(com);
		}
		map.remove(com.getName());
	}

	public static List allWhoIDependOn(MetaData node) {
		List<GraphNode> result = new ArrayList<GraphNode>();
		List<GraphNode> depends = node.getIDependOn();
		result.addAll(depends);
		if (depends.size() > 0) {
			for (int i = 0; i < depends.size(); i++) {
				List list = Graph.allWhoIDependOn((MetaData) depends.get(i));
				Iterator it = list.iterator();
				while (it.hasNext()) {
					MetaData com = (MetaData) it.next();
					if (!result.contains(com))
						result.add(com);
				}
			}
		}
		return result;
	}

	public static List allWhoDependOnMe(MetaData node) {
		List<GraphNode> result = new ArrayList<GraphNode>();
		List<GraphNode> depends = node.getDependOnMe();
		result.addAll(depends);
		if (depends.size() > 0) {
			for (int i = 0; i < depends.size(); i++) {
				List list = Graph.allWhoDependOnMe((MetaData) depends.get(i));
				Iterator it = list.iterator();
				while (it.hasNext()) {
					MetaData com = (MetaData) it.next();
					if (!result.contains(com))
						result.add(com);
				}
			}
		}
		return result;
	}

	public boolean hasCycle(MetaData node) {
		boolean result = false;
		// If it is already in the set there must be a cycle
		if (cycleSet.contains(node.getName()))
			return true;
		// Add it to the cycle set
		cycleSet.add(node.getName());
		List dlist = node.getIDependOn();
		Iterator it = dlist.iterator();
		while (it.hasNext())
			result = hasCycle((MetaData) it.next());
		// Remove the component from the cycle set
		cycleSet.remove(node.getName());
		return result;
	}

	public boolean isDependentOnPackage(MetaData node, String pkg) {
		List list = allWhoIDependOn(node);
		Iterator it = list.iterator();
		while (it.hasNext()) {
			MetaData com = (MetaData) it.next();
			if (pkg.equals(com.getFileName()))
				return true;
		}
		return false;
	}
}
