/*
 * @(#)file      Module.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  29.4.2005
 * 
 */
package coms.core;

import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import coms.CoMS;

/**
 * Describes functionality which all modules need
 */
public abstract class Module {
	protected MBeanServer server;
	protected String domain;

	public Module() {
		server = CoMS.getServerHandle();
		domain = server.getDefaultDomain();
	}

	/**
	 * Creates a object name
	 * 
	 * @param String
	 *            name of mbean, type of mbean, domain of mbean
	 */
	protected ObjectName createObjectName(String name, String type) {
		try {
			return new ObjectName(domain + ":Name=" + name + ", Type=" + type);
		}
		catch (MalformedObjectNameException e) {
			System.out.println("Malformed object name: " + domain + ":Name="
					+ name + ", Type=" + type);
			e.printStackTrace(System.out);
			System.exit(3);
		}
		return null;
	}

	/**
	 * Creates a object name
	 * 
	 * @param String
	 *            name of mbean, type of mbean, domain of mbean
	 */
	protected ObjectName createObjectName(String d, String name, String type) {
		try {
			return new ObjectName(d + ":Name=" + name + ", Type=" + type);
		}
		catch (MalformedObjectNameException e) {
			System.out.println("Malformed object name: " + d + ":Name=" + name
					+ ", Type=" + type);
			e.printStackTrace(System.out);
			System.exit(3);
		}
		return null;
	}

	/**
	 * Calls the logger mbean to log a message
	 * 
	 * @param String
	 *            describing the message to be logged
	 */
	protected void log(String msg) {
		try {
			ObjectName logger = createObjectName("Logger", "Core");
			server.invoke(logger, "log", new Object[] { msg },
					new String[] { String.class.getName() });
		}
		catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
	}
}
