package coms.core.inference;

import java.util.HashSet;
import java.util.HashMap;
import org.xml.sax.SAXException;

public class Rule{
	
	private String name;
	private String loop = "true";
	private String order = "0";
	
	private HashMap<String, String> selects;
	private HashSet<Condition> conditions;
	private Actions actions;
	
	public Rule(String name){
		this.name = name;
		selects = new HashMap<String, String>();
		conditions = new HashSet<Condition>();
	}
	
	public void setName(String n){
		this.name = n;
	}
	
	public void setLoop(String bol){
		this.loop = bol;
	}
	
	public void setOrder(String o){
		this.order = o;
	}
	
	public void addSelect(String id, String sel){
		this.selects.put(id, sel);
	}
	
	public void addCondition(Condition cond) throws SAXException{
		
		// If left expression is method
		if(cond.isLExpMethod()){
		
			// Get the class id
			String lclassid = cond.getLClassid();
			// Get the class the id belong to
			String lclass = (String) selects.get(lclassid);
			// Check if correct class ids are used
			if(lclass == null)
				throw new SAXException("Unknown class id \""+lclassid+"\" used in condition statement");
				
			// Set class in condition
			cond.setLClass(lclass);
		
		}
		
		// If right expression is method
		if(cond.isRExpMethod()){
			// Get the class id
			String rclassid = cond.getRClassid();
			// Get the class the id belong to
			String rclass = (String) selects.get(rclassid);
			// Check if correct class ids are used
			if(rclass == null)
				throw new SAXException("Unknown class id \""+rclassid+"\" used in condition statement");
			
			// Set the class in condition
			cond.setRClass(rclass);
		}
		
		this.conditions.add(cond);
	}
	
	public void setActions(Actions a){
		this.actions = a;
	}
	
	public String getName(){
		return this.name;
	}
	
	public String allowLoop(){
		return this.loop;
	}
	
	public String getOrder(){
		return this.order;
	}
	
	public HashMap<String, String> getSelects(){
		return this.selects;
	}
	
	public HashSet<Condition> getConditions(){
		return this.conditions;
	}
	
	public Actions getActions(){
		return this.actions;
	}
}