package coms.core.inference;
import java.util.HashSet;
import java.util.Iterator;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import org.xml.sax.SAXException;

public class RuleLoader{
	
	private static String[] keys = {	"#DRULES#", "#DRNAME#", "#DRLOOP#", "#DRSALIENCE#", "#DRDATA#", 
								"#DRPID#", "#DRCNAME#", "#DRCONDITION#", "#DRCONTENT#",	"#DRCCODE#"	};
								
	// Valid xml drools tags
	private static String header 		= 	"<?xml version=\"1.0\" ?>\n";
							
	private static String ruleset		=	"<rule-set name=\"ufo rules\"\n" +
											"\rxmlns=\"http://drools.org/rules\"\n" +
											"\rxmlns:java=\"http://drools.org/semantics/java\"\n" +
											"\rxmlns:xs=\"http://www.w3.org/2001/XMLSchema-instance\"\n" +
											"\rxs:schemaLocation=\"http://drools.org/rules rules.xsd \n" +
											"\rhttp://drools.org/semantics/java java.xsd\">\n" +
											"\r#DRULES#\n"+
											"</rule-set>";
						
	private static String rule 			= 	"\r<rule name=\"#DRNAME#\" no-loop=\"#DRLOOP#\" salience=\"#DRSALIENCE#\">\n" +
											"\r\r#DRDATA#\n" +
											"\r</rule>\n";
	private static String parameter		=	"\r<parameter identifier=\"#DRPID#\"><class>#DRCNAME#</class></parameter>\n";
	private static String condition 	= 	"\r<java:condition>#DRCONDITION#</java:condition>\n";
	private static String consequence 	=	"\r<java:consequence>\n\r\r#DRCONTENT#\n#DRCCODE#\n\r</java:consequence>\n";

	// Gets the xml drools rule header
	private static String getHeader(){
		return header;
	}
	
	// Creates a xml drools rule-set
	private static String createRuleSet(String rules){
		String theset = ruleset;
		theset = theset.replaceFirst(keys[0], rules);
		return theset;	
	}
	
	// Creates a xml drools rule
	private static String createRule(String name, String hasLoop, String salience, String data){
		String therule = rule;
		therule = therule.replaceFirst(keys[1], name);
		therule = therule.replaceFirst(keys[2], hasLoop);
		therule = therule.replaceFirst(keys[3], salience);
		therule = therule.replaceFirst(keys[4], data);
		return therule;
	}
	
	// Creates a xml drools parameter 
	private static String createParameter(String id, String cname){
		String theparam = parameter;
		theparam = theparam.replaceFirst(keys[5], id);
		theparam = theparam.replaceFirst(keys[6], cname);
		return theparam;
	}
	
	// Creates a xml drools condition
	private static String createCondition(String c){
		String thecond = condition;
		
		// Replace '>' and '<' with &gt; and &lt; respectively
		c = c.replace(">", "&gt;");
		c = c.replace("<", "&lt;");
		
		thecond = thecond.replaceFirst(keys[7], c);
		return thecond;
	}
	
	// Creates a xml drools consequence
	private static String createConsequence(String content, String code){
		String thecons = consequence;
			
		thecons = thecons.replaceFirst(keys[8], content);
		thecons = thecons.replaceFirst(keys[9], code);
		return thecons;
	}
	
	// Creates code string
	private static String createCode(HashSet<String> inserts, HashSet<String> updates, HashSet<String> removes){
		String code = "";
		
		// Iterate over action inserts
		Iterator iter = inserts.iterator();
		while(iter.hasNext()){
			String in = (String) iter.next();
			code	+=	"\r\r drools.assertObject("+in+");\n";
		}
		
		// Iterate over action updates
		iter = updates.iterator();
		
		while(iter.hasNext()){
			String upd = (String) iter.next();
			code	+=	"\r\r drools.modifyObject("+upd+");\n";
		}
		
		// Iterate over action removes
		iter = removes.iterator();
		
		while(iter.hasNext()){
			String rm = (String) iter.next();
			
			code	+=	"\r\r drools.retractObject("+rm+");\n";
		}
		
		return code;		
	} 	
    
    public static String buildDroolsXml(RuleSet ruleset){
		HashMap<Integer, Rule> rules = ruleset.getRules();
		
		// Create valid drools rules string
		String xmlstr = getHeader();
		String allrules = "";
		String ruledata = "";
		String code = "";
		
		// Iterate over rules
		Iterator it = rules.keySet().iterator();
		while(it.hasNext()){
			Integer key = (Integer) it.next();
			Rule rule = (Rule) rules.get(key);
			ruledata = "";
			code = "";
			
			// Iterate over selects
			HashMap<String, String> selects = rule.getSelects();
			Iterator iter = selects.keySet().iterator();
			
			while(iter.hasNext()){
				String id = (String) iter.next();
				String classname = (String) selects.get(id);
				
				// Create Parameter
				ruledata	+=	createParameter(id, classname);
				
			}
			
			// Iterate over conditions
			HashSet<Condition> conditions = rule.getConditions();
			iter = conditions.iterator();
			
			while(iter.hasNext()){
				Condition cond = (Condition) iter.next();
				
				// Create condition
				ruledata	+=	createCondition(cond.getCond());
				
			}
			
			// Get actions
			Actions actions = rule.getActions();
			
			// Create code
			code 			+= createCode(actions.getInserts(), actions.getUpdates(), actions.getRemoves());
			
			// Create consequence
			ruledata 		+= createConsequence(actions.getContent(), code);
			
			// Create rule
			allrules		+= createRule(rule.getName(), rule.allowLoop(), rule.getOrder(), ruledata);
		}
			
		xmlstr 	+= createRuleSet(allrules);
			
		return xmlstr;
    }
	
}