package coms.database;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import coms.CoMS;
import coms.core.Config;

public class DatabaseAccess {
	/**
	 * Provides the default type of database connection
	 * 
	 * @return a connection to the database.
	 */
	public static DatabaseConnection getDatabaseConnection(String type)
			throws ConnectionException {
		try {
			MBeanServer server = CoMS.getServerHandle();
			if (server != null) {
				ObjectName db = new ObjectName(server.getDefaultDomain()
						+ ":Name=" + type + ", Type=Core");
				return (DatabaseConnection) server.getAttribute(db,
						"Connection");
			}
			else
				throw new ConnectionException("Could not find mbeansever");
		}
		catch (Exception e) {
			e.printStackTrace();
			String msg = "Could not get attribute \"Connection\" from " + type
					+ " MBean";
			msg += "<br>Caused by: " + e.toString();
			throw new ConnectionException(msg);
		}
	}

	// TODO Hver gang en Insert blir gjort m denne metoden kalles for 
	// notifisere eventhandler om at denne tabellen er modifiesert
	/**
	 * Notifies all components of a table change using the EventHandler.
	 * 
	 * @param String
	 *            table
	 */
	public static void notifyEvent(String table) {
		MBeanServer server = null;
		try {
			// Reads the configuration file of the CoMS to find where the
			// mbeansever is
			HashMap<String, String> config = Config.read("config/config.xml");
			String moduleDomain = config.get("ModuleDomain");
			ArrayList mbeanservers = MBeanServerFactory.findMBeanServer(null);
			Iterator iter = mbeanservers.iterator();
			while (iter.hasNext()) {
				server = (MBeanServer) iter.next();
				if (server.getDefaultDomain().equals(moduleDomain))
					break;
			}
			// If a server is found, choose which type of database we want to
			// contact
			if (server != null) {
				ObjectName componentAgent = new ObjectName(moduleDomain
						+ ":Name=EventHandler, Type=Core");
				server.invoke(componentAgent, "notifyEvent",
						new Object[] { table }, new String[] { String.class
								.getName() });
			}
			else
				throw new ConnectionException("Could not find mbeansever");
		}
		catch (Exception e) {
			// This should not occur
			e.printStackTrace();
		}
	}
}
