package coms.mbeans;

import java.io.File;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import coms.core.JarLoader;
import coms.core.JarLoaderException;

public class ComponentClassRepository implements ComponentClassRepositoryMBean {
	private HashMap<String, Hashtable<String, Class>> packagesDefined = new HashMap<String, Hashtable<String, Class>>();

	public ComponentClassRepository() {}

	public Hashtable<String, Class> getPackageClasses(String serviceName) {
		return (Hashtable<String, Class>) packagesDefined.get(serviceName);
	}

	public void loadFromJar(File jarfile, String serviceName)
			throws JarLoaderException, ClassNotFoundException {
		// Create new jarloader
		JarLoader jarLoader = new JarLoader(jarfile);
		jarLoader.open();
		// Read all classes from jar file
		jarLoader.readAll();
		// Get all classes found
		HashMap<String, byte[]> classes = jarLoader.getClasses();
		jarLoader.close();
		// Update component class loader
		defineAllClasses(classes, serviceName);
	}

	/**
	 * Once we know that the jar loader has data for a class, this method is
	 * called to have the class loaded and defined.
	 * 
	 * @param name
	 *            of the class.
	 * @return Class defined class.
	 */
	private void defineLoadedClass(String name, String serviceName)
			throws ClassNotFoundException {
		Class resultClass = Class.forName(name);
		if (!packagesDefined.containsKey(serviceName)) {
			Hashtable<String, Class> table = new Hashtable<String, Class>();
			table.put(name, resultClass);
			this.packagesDefined.put(serviceName, table);
		}
		else {
			Hashtable<String, Class> table = (Hashtable<String, Class>) packagesDefined
					.get(serviceName);
			table.put(name, resultClass);
		}
	}

	/**
	 * Define all classes found in the jar file.
	 */
	private void defineAllClasses(HashMap<String, byte[]> classes,
			String serviceName) throws ClassNotFoundException {
		// Define classes
		Iterator iterator = classes.keySet().iterator();
		while (iterator.hasNext()) {
			String name = (String) iterator.next();
			this.defineLoadedClass(name, serviceName);
		}
	}
}
