/*
 * @(#)file      EventHandler.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  29.6.2005
 * 
 */
package coms.mbeans;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import javax.management.ObjectName;
import coms.core.Module;

/**
 * Class which provides event handling for components
 */
public class EventHandler extends Module implements EventHandlerMBean {
	HashMap<String, HashSet<ObjectName>> eventMap;

	public EventHandler() {
		eventMap = new HashMap<String, HashSet<ObjectName>>();
	}

	synchronized public void subscribe(String name, String type,
			String version, String table) {
		// If no one has subscribed to table yet, create a new HashSet
		if (!eventMap.containsKey(table))
			eventMap.put(table, new HashSet<ObjectName>());
		// Get the HashSet belonging to a table
		HashSet<ObjectName> set = eventMap.get(table);
		// Create the object name for the component
		ObjectName com = createObjectName(name, type);
		// Add the component to the set and add the set to the table in the
		// hashmap
		set.add(com);
		eventMap.put(table, set);
		System.out.println("Component: " + name + " is subscribed to table "
				+ table);
	}

	synchronized public void notifyEvent(String table) {
		// No components subscribed to this table: return
		if (!eventMap.containsKey(table))
			return;
		// Get subscribed components
		HashSet set = eventMap.get(table);
		Iterator it = set.iterator();
		// Notify all components subscribed to this table
		while (it.hasNext()) {
			ObjectName com = (ObjectName) it.next();
			try {
				// Notify component
				server.invoke(com, "onEvent", new Object[] { table },
						new String[] { String.class.getName() });
			}
			catch (Exception e) {
				e.printStackTrace();
				System.exit(0);
			}
		}
	}
}
