/*
 * @(#)file      InferenceFactory.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.1
 * 
 */
package coms.mbeans;

import javax.management.*;
import javax.management.remote.*;
import java.io.*;
import java.net.MalformedURLException;
import coms.metadata.*;
import coms.core.Module;
import coms.core.inference.Engine;
import coms.core.inference.EngineNotFoundException;
import coms.tools.inference.Fact;
import coms.CoMS;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;


/**
* Class that provides processing components with inference engine functionality
*/
public class InferenceFactory extends Module implements InferenceFactoryMBean{
	
	private String dbtype;
	private HashMap<String, Engine> engineMap;
	
	public InferenceFactory(){
		engineMap = new HashMap<String, Engine>();
	}
	
	public void setDBType(String db){
		this.dbtype = db;
	}
	
	public Engine getEngine(String componentID) throws EngineNotFoundException{
		if(engineMap.containsKey(componentID))
			return (Engine) engineMap.get(componentID);
		else
			throw new EngineNotFoundException("The engine with id: "+componentID+" does not exist");
	}
	
	public Engine createEngine(Boolean isPersistent, String componentID){
		Engine engine = new Engine(isPersistent, componentID, dbtype);
		engineMap.put(componentID, engine);
		return engine;
	}
	
	public Hashtable<String, Class> getFactClasses(String componentID){
		try{
			String[] parts = componentID.split("\\.");
			
			// Use CoMS class loader to keep repository of component classes
			ObjectName ccl = createObjectName("ComponentClassRepository", "Core");
			Hashtable<String, Class> table = (Hashtable<String, Class>) server.invoke(ccl, "getPackageClasses", 	
																					new Object[]{parts[0] }, 
																					new String[]{String.class.getName()});
																					
			Hashtable<String, Class> facttable = new Hashtable<String, Class>();
			
			// Check each class if it implements fact interface
			Iterator it = table.keySet().iterator();
			while(it.hasNext()){
				String key = (String) it.next();
				Class cls = (Class) table.get(key);
				Class[] interfaces = cls.getInterfaces();
				
				for(int i=0;i<interfaces.length;i++){
					//System.out.println(interfaces[i].getName());
					if(interfaces[i].getName().equals(Fact.class.getName()))
						facttable.put(key, cls);
				}
			}
			
			return facttable;																	
			
		}catch(Exception e){
			e.printStackTrace();
			System.exit(0);
		}
		return null;
	}
}