/*
 * @(#)file      JarUploader.java
 * @(#)author    Mats Mortensen
 * @(#)version   0.9
 * @(#)lastedit  25.4.2005
 *
 */
package coms.mbeans;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.management.ObjectName;
import coms.core.Module;

/**
 * Class that enables upload and deploying of jar files to the CoMS
 */
public class JarUploader extends Module implements JarUploaderMBean {
	private String deployDir;

	public void setDeployDir(String dir) {
		deployDir = dir;
	}

	public static byte[] BytesTobytes(Byte[] data) {
		byte buf[] = new byte[(int) data.length];
		for (int i = 0; i < data.length; i++)
			buf[i] = data[i].byteValue();
		return buf;
	}

	public String uploadFile(Byte[] filedata, String filename) {
		try {
			String path = deployDir + filename;
			String result = "";
			File file = new File(path);
			String extension = extractExtension(file.getName());
			// Deploy one jar file
			if (extension.equals("jar")) {
				byte[] buf = BytesTobytes(filedata);
				writeBytes(path, buf);
				if (loadJar(file))
					result += deployJars();
				// Extract a zip archive with jar files and deploy all
			}
			else if (extension.equals("zip")) {
				byte[] buf = BytesTobytes(filedata);
				ArrayList filelist = extractZipArchive(buf, deployDir);
				Iterator it = filelist.iterator();
				it = filelist.iterator();
				// Load jar file
				boolean success = true;
				while (it.hasNext())
					if (!loadJar(new File(deployDir + (String) it.next())))
						success = false;
				if (success)
					result += deployJars();
			}
			else {
				deleteJarFromSystem(file);
				result = "<span style=\"color:red\">Error: </span>The file: "
						+ file.getName()
						+ " has been removed from the system because it is not a valid jar or zip file";
				log(result);
			}
			return result;
		}
		catch (IOException e) {
			String result = "Could not receive the jar file \"" + filename
					+ "\"";
			log(result);
			deleteJarFromSystem(filename);
			return result;
		}
	}

	private void writeBytes(String filename, byte[] buf) throws IOException {
		BufferedOutputStream output = new BufferedOutputStream(
				new FileOutputStream(filename));
		output.write(buf, 0, buf.length);
		output.flush();
		output.close();
	}

	private ArrayList extractZipArchive(byte[] filedata, String path)
			throws IOException {
		ByteArrayInputStream bais = new ByteArrayInputStream(filedata);
		BufferedInputStream bis = new BufferedInputStream(bais);
		ZipInputStream zis = new ZipInputStream(bis);
		ZipEntry ze = null;
		ArrayList<String> list = new ArrayList<String>();
		while ((ze = zis.getNextEntry()) != null) {
			if (!ze.isDirectory()) {
				int size = (int) ze.getSize();
				if (size != -1) {
					byte[] data = new byte[(int) size];
					int rb = 0;
					int chunk = 0;
					while (((int) size - rb) > 0) {
						chunk = zis.read(data, rb, (int) size - rb);
						if (chunk == -1) {
							break;
						}
						rb += chunk;
					}
					String filepath = path + ze.getName();
					writeBytes(filepath, data);
					list.add(ze.getName());
				}
			}
		}
		zis.close();
		bis.close();
		return list;
	}

	private String deleteJarFromSystem(File file) {
		String result = "";
		if (file.exists()) {
			if (file.delete())
				result += "The file: " + file.getName()
						+ " has been removed from the system<br>";
			else
				result += "<span style=\"color:red\">Error: </span>The file: "
						+ file.getName() + " could not be deleted<br>";
		}
		else {
			result += "<span style=\"color:red\">Error: </span>Could not delete "
					+ file.getName() + " because it does not exist<br>";
		}
		return result;
	}

	public void deleteJarFromSystem(String filename) {
		String filepath = deployDir + filename;
		String result = "";
		File file = new File(filepath);
		result += deleteJarFromSystem(file);
		log(result);
	}

	private Boolean loadJar(File file) {
		try {
			ObjectName loader = createObjectName("ComponentLoader", "Core");
			return (Boolean) server.invoke(loader, "loadJarFile",
					new Object[] { file },
					new String[] { File.class.getName() });
		}
		catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		return null;
	}

	private String deployJars() {
		try {
			ObjectName loader = createObjectName("ComponentLoader", "Core");
			return (String) server.invoke(loader, "deployLoadedJars", null,
					null);
		}
		catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		return null;
	}

	private String extractExtension(String filename) {
		String filenameExtension = filename;
		int extension = filenameExtension.indexOf(".");
		if (extension == -1)
			return "";
		return filenameExtension.substring(extension + 1, filename.length())
				.toLowerCase();
	}
}
