package coms.mbeans;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import coms.core.LogObject;
import coms.core.Module;

/**
 * This class has the logging methods
 * 
 * @author Mats Mortensen, Timo Schalkowski
 */
public class Logger extends Module implements LoggerMBean {
	// Options
	private boolean isOn = true;
	private boolean printToFile = true;
	private boolean printToConsole = false;
	private String file;
	// Private
	private PrintStream logFile;
	private SimpleDateFormat date = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	private ArrayList<MsgObject> statusMsg = new ArrayList<MsgObject>();
	Long oldestmsg = new Long(0);

	/**
	 * Sets the file attribute.
	 * 
	 * @param n
	 *            String
	 */
	public void setFile(String n) {
		file = n;
	}

	/**
	 * Clears the logfile.
	 */
	public void clear() {
		try {
			logFile = new PrintStream(new FileOutputStream(file, false));
			logFile.print("");
			clearMsg();
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Sets the isOn attribute
	 * 
	 * @param i
	 *            boolean
	 */
	public void setOn(boolean i) {
		isOn = i;
	}

	/**
	 * Returns the state of the isOn boolean attribute
	 * 
	 * @return true or false
	 */
	public boolean isOn() {
		return isOn;
	}

	/**
	 * Writes a string to the logfile and logarray
	 * 
	 * @param msg
	 *            A string message
	 */
	public void log(String msg) {
		try {
			if (isOn) {
				String logmsg = "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">";
				logmsg += "<tr><td><p><span style=\"font-family: Times;font-size:10px;color:#0000fff\">"
						+ date.format(new Date()) + "</span></td></tr>";
				logmsg += "<tr><td><span style=\"font-family: Times;font-size:10px\">"
						+ " " + msg + "</span></p></td></tr></table>";
				if (printToFile) {
					logFile = new PrintStream(new FileOutputStream(file, true));
					logFile.println(logmsg);
					logFile.close();
				}
				if (printToConsole)
					System.out.println(logmsg);
				updateMsgBuffer(logmsg);
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Updates the buffer used by the client to get status msgs
	 * 
	 * @param String
	 *            msg
	 */
	private void updateMsgBuffer(String msg) {
		long time = System.currentTimeMillis();
		MsgObject msgobj = new MsgObject(msg, time);
		statusMsg.add(msgobj);
		oldestmsg = time;
	}

	/**
	 * Returns the log msgs not recieved yet
	 * 
	 * @return last log
	 */
	public synchronized LogObject readLog(Long time) {
		ArrayList<String> newmsgs = new ArrayList<String>();
		Iterator it = statusMsg.iterator();
		while (it.hasNext()) {
			MsgObject msg = (MsgObject) it.next();
			if (time < msg.time)
				newmsgs.add(msg.msg);
		}
		LogObject logobj = new LogObject(oldestmsg, newmsgs);
		return logobj;
	}

	/**
	 * Checks if there are new messages
	 * 
	 * @param Long
	 *            time
	 * @return Boolean true/false
	 */
	public synchronized Boolean checkLog(Long time) {
		if (time >= oldestmsg)
			return false;
		else
			return true;
	}

	/**
	 * Clears the last log array.
	 */
	public void clearMsg() {
		statusMsg.clear();
	}
}

class MsgObject {
	String msg;
	long time;

	public MsgObject(String msg, long time) {
		this.msg = msg;
		this.time = time;
	}
}
