package coms.metadata;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

public class DeploymentDescWriter {
	static FileWriter file;
	static DeploymentDescription ddesc;

	private static void writeHeader() throws IOException {
		file
				.write("<?xml version = \"1.0\" encoding=\"UTF-8\" standalone = \"no\"?>\n");
		file
				.write("<!DOCTYPE METAINFORMATION SYSTEM \"./config/DeploymentDescriptor_DTD.dtd\">\n");
		file.write("<METAINFORMATION>\n");
	}

	private static void writeService() throws IOException {
		Service service = ddesc.getServiceDescription().getService();
		HashMap operations = service.getOperationMap();
		file.write("<service>\n");
		file.write("\t<name>" + service.getName() + "</name>\n");
		file.write("\t<version>" + service.getVersion() + "</version>\n");
		file.write("\t<description>" + service.getDescription()
				+ "</description>\n");
		file.write("\t<dbType>" + service.getDbType() + "</dbType>\n");
		if (operations != null) {
			if (!operations.isEmpty()) {
				Set keyset = operations.keySet();
				Iterator it = keyset.iterator();
				file.write("\t<operation>\n");
				while (it.hasNext()) {
					String key = (String) it.next();
					ServiceOperation operation = (ServiceOperation) operations
							.get(key);
					file.write("\t\t<display_name>" + operation.getName()
							+ "</display_name>\n");
					file.write("\t\t<cname>" + operation.getComponentName()
							+ "</cname>\n");
					file.write("\t\t<ctype>" + operation.getComponentType()
							+ "</ctype>\n");
					file.write("\t\t<method>" + operation.getMethod()
							+ "</method>\n");
				}
				file.write("\t</operation>\n");
			}
		}
		file.write("</service>\n");
	}

	private static void writeDeploy() throws IOException {
		LinkedHashMap components = ddesc.getComponentMap();
		file.write("<deploy>\n");
		Set keyset = components.keySet();
		Iterator it = keyset.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			file.write("\t<component>\n");
			MetaData data = (MetaData) components.get(key);
			file.write("\t\t<name>" + data.getName() + "</name>\n");
			file.write("\t\t<class>" + data.getClassName() + "</class>\n");
			file.write("\t\t<metafile>" + data.getMetaFileName()
					+ "</metafile>\n");
			file.write("\t</component>\n");
		}
		file.write("</deploy>\n");
	}

	private static void writeFooter() throws IOException {
		file.write("</METAINFORMATION>");
	}

	public static void write(DeploymentDescription data, File dir) {
		try {
			ddesc = data;
			File metadataFile = new File(dir, "DeploymentDescriptor.xml");
			metadataFile.createNewFile();
			file = new FileWriter(metadataFile);
			writeHeader();
			writeService();
			writeDeploy();
			writeFooter();
			file.close();
		}
		catch (IOException e) {
			System.out.println("Theres something going on...");
		}
	}
}
