package coms.metadata;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class IConfigMetaDataWriter {
	static File metadataFile;
	static FileWriter file;

	public static void writeHeader() throws IOException {
		file
				.write("<?xml version = \"1.0\" encoding=\"UTF-8\" standalone = \"no\"?>\n");
		file
				.write("<!DOCTYPE METAINFORMATION SYSTEM \"./config/ConfigMeta_DTD.dtd\">\n");
		file.write("<METAINFORMATION>\n<component>\n");
	}

	public static void writeBody(MetaData data) throws IOException {
		file.write("\t<version>" + data.getVersion() + "</version>\n");
		file.write("\t<type>" + data.getType() + "</type>\n");
		file.write("\t<description>" + data.getDescription()
				+ "</description>\n");
		file.write("\t<bindingtype>" + data.getBindingType()
				+ "</bindingtype>\n");
	}

	public static void writeBindings(MetaData data) throws IOException {
		Set set = data.getBindings();
		Iterator it = set.iterator();
		while (it.hasNext()) {
			Bind binding = (Bind) it.next();
			file.write("\t<bind comp=\"" + binding.getComp() + "\" to=\""
					+ binding.getTo() + "\" class=\""
					+ binding.getBindingClass() + "\"/>\n");
		}
	}

	public static void writeDependencies(MetaData data) throws IOException {
		List deplist = data.getDependenciesList();
		if (!deplist.isEmpty()) {
			Iterator it = data.getDependencies();
			file.write("\t<dependencies>\n");
			while (it.hasNext()) {
				String dep = (String) it.next();
				file.write("\t\t<dname>" + dep + "</dname>\n");
			}
			file.write("\t</dependencies>\n");
		}
	}

	public static void writeSubscriptions(MetaData data) throws IOException {
		Set list = data.getSubscriptions();
		Iterator it = list.iterator();
		if (it.hasNext()) {
			file.write("\t<subscribe>");
			while (it.hasNext()) {
				String table = (String) it.next();
				file.write("\t\t<to_table>" + table + "</to_table>\n");
			}
			file.write("\t</subscribe>");
		}
	}

	public static void writeConfigOptions(MetaData data) throws IOException {
		HashMap<String, String> customOptions = data.getCustomOptions();
		// If this component has config options^
		if (customOptions != null) {
			if (!customOptions.isEmpty()) {
				Set keyset = customOptions.keySet();
				Iterator it = keyset.iterator();
				file.write("\t<config>\n");
				while (it.hasNext()) {
					String key = (String) it.next();
					String value = (String) customOptions.get(key);
					if (key.length() > 0) {
						file.write("\t\t<option>\n");
						file.write("\t\t\t<optionname>" + key
								+ "</optionname>\n");
						file.write("\t\t\t<optionvalue>" + value
								+ "</optionvalue>\n");
						file.write("\t\t</option>\n");
					}
				}
				file.write("\t</config>\n");
			}
		}
	}

	public static void writeFooter() throws IOException {
		file.write("</component>\n</METAINFORMATION>");
	}

	public static void write(MetaData data, FileWriter f) throws IOException {
		file = f;
		// SETS THE HEADER
		writeHeader();
		// SETS THE BODY
		writeBody(data);
		// writes out the bindings
		writeBindings(data);
		// Writes out dependencies
		writeDependencies(data);
		// Writes out subscriptions
		writeSubscriptions(data);
		// Writes out configuration parameters
		writeConfigOptions(data);
		// SETS THE FOOTER
		writeFooter();
	}
}
