package coms.metadata;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import coms.core.GraphNode;

public class MetaData extends GraphNode {
	private String service;
	private String id;
	private String name;
	private String version;
	private String type;
	private boolean activated;
	private String dbType; // Tells which type of database it is using
	private String classname; // Name of the class file
	private Class componentClass;
	private String filename; // Name of the jar file.
	private String metafilename; // Name of the meta file
	private String description; // Description of the component
	private String bindingtype; // The binding type to be used
	private ArrayList<Table> tables; // The tables that this component is
	// using
	private ArrayList<String> dependencies;
	public HashMap<String, String> customOptions; // All options specific to a
	// component
	private HashSet<String> subscriptions; // All tables this component is
	// subscribed to
	private HashSet<Bind> bindSet; // All bindings for component

	public MetaData() {
		super();
		tables = new ArrayList<Table>();
		dependencies = new ArrayList<String>();
		subscriptions = new HashSet<String>();
		bindSet = new HashSet<Bind>();
	}

	public String getService() {
		return this.service;
	}

	public void setService(String i) {
		this.service = i;
	}

	public String getID() {
		return this.id;
	}

	public void setID(String i) {
		this.id = i;
	}

	public String getClassName() {
		return this.classname;
	}

	public void setClassName(String c) {
		this.classname = c;
	}

	public Class getComponentClass() {
		return this.componentClass;
	}

	public void setComponentClass(Class c) {
		this.componentClass = c;
	}

	public String getFileName() {
		return this.filename;
	}

	public void setFileName(String f) {
		this.filename = f;
	}

	public String getMetaFileName() {
		return this.metafilename;
	}

	public void setMetaFileName(String f) {
		this.metafilename = f;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String d) {
		this.description = d;
	}

	public void setCustomOptions(HashMap<String, String> map) {
		this.customOptions = map;
	}

	public void setCustomOptions(String k, String v) {
		customOptions.put(k, v);
	}

	public HashMap<String, String> getCustomOptions() {
		return customOptions;
	}

	public void addSubscription(String s) {
		this.subscriptions.add(s);
	}

	public HashSet<String> getSubscriptions() {
		return this.subscriptions;
	}

	public String getBindingType() {
		return bindingtype;
	}

	public void setBindingType(String t) {
		this.bindingtype = t;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String n) {
		this.name = n;
	}

	public String getVersion() {
		return this.version;
	}

	public void setVersion(String n) {
		this.version = n;
	}

	public String getType() {
		return this.type;
	}

	public void setType(String n) {
		this.type = n;
	}

	public void setActivated(boolean a) {
		this.activated = a;
	}

	public boolean getActivated() {
		return this.activated;
	}

	public void addTable(Table t) {
		this.tables.add(t);
	}

	public ArrayList getTables() {
		return this.tables;
	}

	public Iterator tableIterator() {
		return this.tables.iterator();
	}

	public void setDbType(String tbt) {
		dbType = tbt;
	}

	public String getDbType() {
		return dbType;
	}

	public void addBinding(Bind b) {
		this.bindSet.add(b);
	}

	public HashSet getBindings() {
		return this.bindSet;
	}

	public void addDependency(String d) {
		this.dependencies.add(d);
	}

	public Iterator getDependencies() {
		return this.dependencies.iterator();
	}

	public ArrayList getDependenciesList() {
		return this.dependencies;
	}

	public boolean equals(MetaData o) {
		if (this.id.equals(o.getID()))
			return true;
		else
			return false;
	}
}
