package coms.metadata;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import coms.core.JarLoader;
import coms.core.JarLoaderException;
import coms.core.JarValidityException;

public class ServiceDescriptorParser implements ContentHandler {
	private JarLoader jarLoader;
	private XMLReader parser;
	private String starttag, endtag;
	private Service service;
	private ServiceDescription serviceDesc;
	private ServiceOperation input;
	private String state = "";
	private String fullvalue = "";
	private HashMap<String, ArrayList<String>> dependencyMap = new HashMap<String, ArrayList<String>>();
	private HashMap<String, Service> serviceMap = new HashMap<String, Service>();

	// Takes appropriate action when we have a starttag of some type
	public void startElement(String namespaceURI, String localName,
			String qName, Attributes atts) throws SAXException {
		starttag = localName.toLowerCase();
		if (starttag.equals("all_services"))
			serviceDesc = new ServiceDescription(jarLoader.getFileName());
		else if (starttag.equals("service"))
			service = new Service();
		if (starttag.equals("depends_on"))
			state = starttag;
		else if (starttag.equals("components"))
			state = starttag;
		else if (starttag.equals("user_input")) {
			state = starttag;
			input = new ServiceOperation();
		}
	}

	// Takes appropriate action when we have a endtag of come type
	public void endElement(String namespaceURI, String localName, String qName)
			throws SAXException {
		endtag = localName.toLowerCase();
		if (endtag.equals("name") && state.equals("")) {
			service.setName(fullvalue);
			serviceMap.put(service.getName(), service);
		}
		if (endtag.equals("dservice") && state.equals("depends_on"))
			buildDependencyMap();
		if (endtag.equals("description"))
			service.setDescription(fullvalue);
		if (endtag.equals("name") && state.equals("components"))
			service.addComponent(fullvalue);
		if (endtag.equals("display_name") && state.equals("user_input"))
			input.setName(fullvalue);
		if (endtag.equals("cname") && state.equals("user_input"))
			input.setComponentName(fullvalue);
		if (endtag.equals("ctype") && state.equals("user_input"))
			input.setComponentType(fullvalue);
		if (endtag.equals("method") && state.equals("user_input"))
			input.setMethod(fullvalue);
		if (endtag.equals("service"))
			serviceDesc.setService(service);
		if (endtag.equals("depends_on"))
			state = "";
		else if (endtag.equals("components"))
			state = "";
		else if (endtag.equals("user_input")) {
			state = "";
			service.addOperation(input);
		}
		starttag = "";
		fullvalue = "";
		endtag = "";
	}

	// Makes a hashmap where the services are keys and their dependencies are
	// list values
	private void buildDependencyMap() {
		if (!dependencyMap.containsKey(service.getName())) {
			ArrayList<String> newlist = new ArrayList<String>();
			newlist.add(fullvalue);
			dependencyMap.put(service.getName(), newlist);
		}
		else {
			ArrayList<String> list = dependencyMap.get(service.getName());
			list.add(fullvalue);
			dependencyMap.put(service.getName(), list);
		}
	}

	// Gets the content between the start and end tag
	public void characters(char[] text, int start, int length)
			throws SAXException {
		String value = new String(text, start, length);
		value = value.trim();
		fullvalue += value;
	}

	public ServiceDescriptorParser(JarLoader jl) throws SAXException {
		try {
			jarLoader = jl;
			parser = XMLReaderFactory.createXMLReader();
			parser.setFeature("http://xml.org/sax/features/validation", true);
		}
		catch (SAXNotRecognizedException ex) {
			System.out.println("Installed XML parser cannot validate");
			System.exit(0);
		}
		catch (SAXNotSupportedException e) {
			System.out.println("Installed XML parser cannot validate");
			System.exit(0);
		}
		// Install the content handler
		parser.setContentHandler(this);
	}

	// Initiate parsing of xml document
	public ServiceDescription parse(String xmlfile) throws SAXParseException,
			SAXException, JarLoaderException, JarValidityException {
		try {
			// Set error handler
			parser.setErrorHandler(new ValidityErrorHandler(xmlfile));
			// Read the Service Descriptor
			InputStream stream = jarLoader.readXMLFile(xmlfile);
			// Parse xml file
			parser.parse(new InputSource(stream));
			// Add dependency Map
			serviceDesc.setDependencyMap(dependencyMap);
			return serviceDesc;
		}
		catch (IOException e) {
			throw new JarLoaderException("Error reading xml data from: "
					+ xmlfile + " when parsing. \nCaused by: " + e.getMessage());
		}
	}

	public void setDocumentLocator(Locator locator) {}

	public void startDocument() throws SAXException {}

	public void endDocument() throws SAXException {}

	public void ignorableWhitespace(char[] text, int start, int length)
			throws SAXException {}

	public void processingInstruction(String target, String data)
			throws SAXException {}

	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {}

	public void endPrefixMapping(String prefix) throws SAXException {}

	public void skippedEntity(String name) throws SAXException {}
}
