package coms.metadata;

public class TableField {
	private String name;
	private String type;
	private String def;
	private boolean isPrimaryKey;

	/**
	 * Creates an empty table field.
	 */
	public TableField() {
		this("", "", "", false);
	}

	/**
	 * Creates a table field to hold a field name, field type and a default
	 * value
	 * 
	 * @param n
	 *            the name of the table field
	 * @param t
	 *            the type of the table field
	 * @param d
	 *            the default value of the table field
	 */
	public TableField(String n, String t, String d) {
		this(n, t, d, false);
	}

	/**
	 * Creates a table field to hold a field name, field type and a default
	 * value
	 * 
	 * @param n
	 *            the name of the table field
	 * @param t
	 *            the type of the table field
	 * @param d
	 *            the default value of the table field
	 * @param pk
	 *            wheater this field is the primary key
	 */
	public TableField(String n, String t, String d, boolean pk) {
		name = n.toUpperCase();
		type = t.toUpperCase();
		if (d != null)
			def = d.toUpperCase();
		else
			def = "";
		isPrimaryKey = pk;
	}

	/**
	 * Sets the name of the table field
	 * 
	 * @param n
	 *            the new name of the table field
	 */
	public void setName(String n) {
		name = n.toUpperCase();
	}

	/**
	 * Gives access to the name of the table field
	 * 
	 * @return the name of the table field
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the type of the table field
	 * 
	 * @param t
	 *            the new type of the table field
	 */
	public void setType(String t) {
		type = t.toUpperCase();
	}

	/**
	 * Gives access to the type of the table field
	 * 
	 * @return the type of the table field
	 */
	public String getType() {
		return type;
	}

	/**
	 * Sets the default value of the table field
	 * 
	 * @param d
	 *            the new default value of the table field
	 */
	public void setDefault(String d) {
		def = d.toUpperCase();
	}

	/**
	 * Gives access to the default value of the table field
	 * 
	 * @return the default value of the table field
	 */
	public String getDefault() {
		return def;
	}

	/**
	 * Tells wheater this field is the primary key
	 * 
	 * @return wheater it is the primary key
	 */
	public boolean isPrimaryKey() {
		return isPrimaryKey;
	}
}
