package coms.tools.relational;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import coms.CoMS;
import coms.database.ConnectionException;
import coms.database.DatabaseConnection;

/**
 * Select is used against relational databases to select a piece of data.
 * 
 * @author Frank, based on coms.tools.relational.Input.java
 * 
 */
public class Update {
	public static final int EQUAL = 0;
	public static final int NOT = 1;
	public static final int SMALLER = 2;
	public static final int GREATER = 3;
	public static final int LIKE = 4;
	public static final int MAX = 5;
	public static final int MIN = 6;
	public static final int ANY = 7;
	public static final int ASCENDING = 0;
	public static final int DESCENDING = 1;
	private String tbl;
	private String dbType;
	private String order = "";
	
	private String prevCol = null;

	private Object prevVal = null;

	private HashMap<String, Object> data = new HashMap<String, Object>();

	private ArrayList<String> column = new ArrayList<String>();
	private ArrayList<Object> params = new ArrayList<Object>();
	private ArrayList<String> cond = new ArrayList<String>();

	public Update(String tableName, String DBtype) {
		this.tbl = tableName;
		this.dbType = DBtype;
	}
//
//	/**
//	 * Sets a column in the table to be selected
//	 * 
//	 * @param col
//	 *            Name of column to be selected
//	 */
//	public void column(String col) {
//		column.add(col);
//	}
//
//	public void column(String col, int condition) {
//		if (condition == MAX)
//			column.add(" MAX(" + col + ")");
//		else if (condition == MIN)
//			column.add(" MIN(" + col + ")");
//	}

	/**
	 * Inserts the value into the column
	 * 
	 * @param col
	 *            column where the value is to be placed
	 * @param value
	 *            to be put into the database
	 */
	public void column(String col, Object value) {
		prevCol = col;
		prevVal = data.get(col);
		data.put(col, value);
	}
	
	/**
	 * Adds another value to the sql string
	 * 
	 * @param field
	 *            the field to be compared against a value
	 * @param condition
	 *            the compare condition. Can be either: Constant.equal,
	 *            Constant.not, Constant.greater, Constant.smaller
	 * @param value
	 *            the value field is compared to
	 */
	public void and(String field, int condition, Object value) {
		setCondition(field, condition, value, " AND");
	}

	/**
	 * Puts this condition as a or statement
	 * 
	 * @param field
	 *            the field to be compared against a value
	 * @param condition
	 *            the compare condition. Can be either: Constant.equal,
	 *            Constant.not, Constant.greater, Constant.smaller
	 * @param value
	 *            the value field is compared to
	 */
	public void or(String field, int condition, Object value) {
		setCondition(field, condition, value, " OR");
	}

	/**
	 * Will not select when this condition is true
	 * 
	 * @param field
	 *            the field to be compared against a value
	 * @param condition
	 *            the compare condition. Can be either: Constant.equal,
	 *            Constant.not, Constant.greater, Constant.smaller
	 * @param value
	 *            the value field is compared to
	 */
	public void andNot(String field, int condition, Object value) {
		setCondition(field, condition, value, " AND NOT");
	}

	/**
	 * Will not select when this condition is true
	 * 
	 * @param field
	 *            the field to be compared against a value
	 * @param condition
	 *            the compare condition. Can be either: Constant.equal,
	 *            Constant.not, Constant.greater, Constant.smaller
	 * @param value
	 *            the value field is compared to
	 */
	public void orNot(String field, int condition, Object value) {
		setCondition(field, condition, value, " OR NOT");
	}

	/**
	 * Chooses which way the result will be ordered
	 * 
	 * @param col
	 *            the column the result will be ordered by
	 * @param dir
	 *            The direction of the result. Either Select.ASCENDING or
	 *            Select.DECENDING
	 */
	public void orderBy(String col, int dir) {
		if (dir == ASCENDING) {
			order = " ORDER BY " + col + " ASC";
		}
		else if (dir == DESCENDING) {
			order = " ORDER BY " + col + " DESC";
		}
	}

	/**
	 * Undoes the last condition set
	 */
	public void undo() {
		cond.remove(cond.size() - 1);
		params.remove(params.size() - 1);
	}

	/**
	 * Will execute the options put into select against a relational database.
	 * 
	 * @throws SQLException
	 *             if there is something wrong in the generated SQL string
	 * @throws ConnectionException
	 *             if a connection cannot be aquired
	 */
	public void execute() throws SQLException, ConnectionException {
		
//		List<Map<String, Object>> result = null;

		
		// Combines the different elements to a sql string
		String sqlStr = "UPDATE " + tbl + " SET ";
		
		Iterator it = data.keySet().iterator();
		// Iterates over the data picking out the fields in each row
		while (it.hasNext()) {
			// Drops the commas the last time
			sqlStr += it.next() + " = ?" + (it.hasNext() ? "," : "");

		}

		// If there are conditions, add them to the string
		if (!cond.isEmpty()) {
			sqlStr += " WHERE";
			it = cond.iterator();
			sqlStr += (String) it.next();
			while (it.hasNext())
				sqlStr += (String) it.next();
		}
		
//		for (int i = 0; i < params.size(); i++) {
//			data.put("cond" + i, params.get(i));
//		}
		
//		// Adds the chosen ordering if any
//		sqlStr += order;
		try {
			System.out.println(sqlStr);
//			Object[] objs = data.values().toArray();
//			for(int i=0;i<objs.length;i++)
//				System.out.print(objs[i].toString() + (i == objs.length-1 ? "\n" : " "));
			
			// Executes the sql string on a database connection
			DatabaseConnection con = getDatabaseConnection(dbType);
//			result = con.executeSQL(sqlStr, params.toArray(), true);
			con.executeSQL(sqlStr, data.values().toArray(), false);
			con.release();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
//		return result;
	}

	private DatabaseConnection getDatabaseConnection(String type)
			throws Exception {
		MBeanServer server = CoMS.getServerHandle();
		ObjectName db = new ObjectName(server.getDefaultDomain() + ":Name="
				+ type + ", Type=Core");
		return (DatabaseConnection) server.getAttribute(db, "Connection");
	}

	/**
	 * Creates a substring containing the information in this condition
	 * 
	 * @param field
	 *            in the database to be quaried against
	 * @param condition
	 *            which boolean expression to be used in the checking
	 * @param value
	 *            to be checked against
	 * @param type
	 *            can be either AND, OR or NOT
	 */
	private void setCondition(String field, int condition, Object value,
			String type) {
		
		String sqlStr = "";
		// If this is the first condition, don't use prefix AND or OR
		if (cond.size() == 0) {
			// If the first condition is a NOT, drop AND and OR, but keep NOT
			if (type.equals(" AND NOT") || type.equals(" OR NOT")) {
				sqlStr = " NOT";
			}
		}
		else
			sqlStr = type;
		sqlStr += " " + field;
		if (condition == GREATER)
			sqlStr += " > " + (value.toString().indexOf(" ") != -1 ? "'" + value + "'" : value);
		else if (condition == SMALLER)
			sqlStr += " < " + (value.toString().indexOf(" ") != -1 ? "'" + value + "'" : value);
		else if (condition == NOT)
			sqlStr += " != " + (value.toString().indexOf(" ") != -1 ? "'" + value + "'" : value);
		else if (condition == LIKE)
			sqlStr += " LIKE " + (value.toString().indexOf(" ") != -1 ? "'" + value + "'" : value);
		else
			sqlStr += " = " + (value.toString().indexOf(" ") != -1 ? "'" + value + "'" : value);
		cond.add(sqlStr);
		params.add(value);
	}
}
