package com.telenor.apms.fifamos.client.j2se.utils;

/**
 * RPCHandler.java
 * 
 * Class that is responsible for making the web service calls, and return results as vectors
 * 
 * @author JAG
 * @copyright JAG  2006
 * @version 1.0
 */

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;


import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

import com.telenor.apms.fifamos.client.j2se.objects.Frame;
import com.telenor.apms.fifamos.client.j2se.objects.SeaCage;
import com.telenor.apms.fifamos.client.j2se.objects.Sensor;
import com.telenor.apms.fifamos.client.j2se.utils.SaveFile;

/**
 * @author Jon Arne
 *
 */
public class RPCHandler {
	
	
	private long[][] sensordataTable;

	public Vector<Frame> getAllFrames() throws Exception{
		Vector<Frame> res = new Vector<Frame>();
		Vector<String> params = new Vector<String>();
		params.add("");
		params.add("");
		params.add("");
		params.add("");
		debug("getAllFrames contacting web service..");
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		debug(Constants.SERVER_URL);
		String response = (String)client.execute ("Output.getFrames", params);
		new SaveFile(response);
		StringTokenizer st = new StringTokenizer(response, ";");
		while(st.hasMoreTokens()) {
			res.addElement(new Frame(st.nextToken()));
		}
		return res;
	}
	
	public Vector<SeaCage> getAllSeaCages(String frameId) throws Exception{
		Vector<SeaCage> res = new Vector<SeaCage>();
		Vector<String> params = new Vector<String>();
		params.addElement(frameId);
		debug("getAllSeaCages contacting web service..");
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		String response = (String)client.execute ("Output.getSeaCagesForFrame", params);
		new SaveFile(response);
		StringTokenizer st = new StringTokenizer(response, ";");
		while(st.hasMoreTokens()) {
			res.addElement(new SeaCage(st.nextToken()));
		}
		return res;
	}
	
	public Vector getAllSensors(String seaCageID) throws Exception {
		Vector<Sensor> res = new Vector<Sensor>();
		Vector<String> params = new Vector<String>();
		params.addElement(seaCageID);
		debug("getsensors for seacageid: "+seaCageID);
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		String response = (String)client.execute ("Output.getSensorsForSeaCage", params);
		StringTokenizer st = new StringTokenizer(response, ";");
		while(st.hasMoreTokens()) {
			res.addElement(new Sensor(st.nextToken()));
		}
		return res;
	}

	public long[][]getHistoryForSensor(int sensorID, long period) {
		try {
			long time = new Date().getTime();
			Vector<String> params = new Vector<String>();
			debug("SensorGraph contacting web service..");
			XmlRpcClient client = new XmlRpcClient(
					Constants.SERVER_URL);
			params.addElement(String.valueOf(sensorID));
			params.addElement(String.valueOf(period));
			
			String sensordata = (String) client.execute(
					"Output.getHistoryForSensor", params);
			time = new Date().getTime()-time;
			StringVector sv = new StringVector(sensordata, ';');
			StringVector sv2 = new StringVector(sv.stringElementAt(0),':');
			sensordataTable = new long[sv.size()-1][2];
			
//			minX = Long.parseLong(sv2.stringElementAt(0));//henter ut periodens frste timestamp
//			maxX = Long.parseLong(sv2.stringElementAt(1));//henter ut periodens siste timestamp
//			debug("X-Aksen: " + minX + ", " + maxX);
			for(int i = 1; i<sv.size(); i++) {//frste element hoppes over. Her ligger kun timestamp som angir start og slutt av periode
				sv2 = new StringVector(sv.stringElementAt(i),':');

				for (int j = 0; j < sv2.size(); j++) {
					sensordataTable[i-1][j] = (long)Double.parseDouble(sv2.stringElementAt(j));
				}
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		return sensordataTable;		
	}
	
	//Add methods
	public String addFrame(Frame frame) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"add\">"+frame.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}
	
	public String addSeaCage(SeaCage sc) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"add\">"+sc.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}
	
	public String addSensor(Sensor sensor) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"add\">"+sensor.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}
	
	//Remove methods
	public String removeFrame(String frameID) throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"remove\"><frame><id>" + frameID + "</id></frame></command></xml>";
		params.add(xml);
		return (String)client.execute("Input.postConsumerXml", params);
	}

	public String removeSeaCage(String seaCageID) throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"remove\"><seacage><id>" + seaCageID + "</id></seacage></command></xml>";
		params.add(xml);
		return (String)client.execute("Input.postConsumerXml", params);
	}

	public String removeSensor(String sensorID) throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"remove\"><sensor><id>" + sensorID + "</id></sensor></command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}

	//Update methods
	public String updateSensor(Sensor sensor) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"update\">"+sensor.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}
	
	public String updateFrame(Frame frame) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"update\">"+frame.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}
	
	public String updateSeaCage(SeaCage seaCage) throws XmlRpcException, IOException {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String xml = "<xml><command type=\"update\">"+seaCage.getXml()+"</command></xml>";
		params.add(xml);
		return (String) client.execute("Input.postConsumerXml", params);
	}

	
	public String confirmAlarms() throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		return (String)client.execute("Input.confirmAlarms", params);
	}

	private void debug(String string) {
		System.out.println("RPCHandler debug: "+string);
	}

	public String getFishFarmAsXml() throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		String res = (String)client.execute("Output.getFishFarmAsXml", params);
		System.err.println(res.length());
		new SaveFile(res);
		return res;
	}

	public String getSensorTypes() throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		return (String)client.execute("Output.getSensorTypes", params);
	}

	public String cleanDB() throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		return (String)client.execute("Input.cleanDB", params);		
	}

	public String fillDB(int frames, int seaCages) throws Exception {
		XmlRpcClient client = new XmlRpcClient (Constants.SERVER_URL);
		Vector<String> params = new Vector<String>();
		params.add(""+frames);
		params.add(""+seaCages);
		return (String)client.execute("Input.fillDB", params);		
	}

}
