package com.telenor.apms.fifamos;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import uk.me.jstott.jcoord.LatLng;

import com.telenor.apms.fifamos.objects.Command;
import com.telenor.apms.fifamos.objects.Frame;
import com.telenor.apms.fifamos.objects.GPS;
import com.telenor.apms.fifamos.objects.SeaCage;
import com.telenor.apms.fifamos.objects.Sensor;
import com.telenor.apms.fifamos.objects.SensorType;
import com.telenor.apms.fifamos.objects.SensorValue;
import com.telenor.apms.fifamos.utils.Convert;
import com.telenor.apms.fifamos.utils.XMLEventHandler;
import com.telenor.apms.fifamos.utils.XMLEventListener;
import coms.component.InputComponent;
import coms.database.ConnectionException;
import coms.tools.relational.Delete;
import coms.tools.relational.Insert;
import coms.tools.relational.Select;
import coms.tools.relational.Update;

/**
 * @author Frank
 *
 */
public class Input extends InputComponent implements InputMBean,
		XMLEventListener {

	private XMLReader parser;


	private boolean outofbounds;

	/*
	 * @Override (non-Javadoc)
	 *
	 * @see coms.component.Component#init()
	 */
	public void init() {
		try {
			// engine = createEngine(false);
			// engine.loadRules(getExtDir()+"fifamo.drl");

			parser = XMLReaderFactory.createXMLReader();
			parser.setContentHandler(new XMLEventHandler(this));

//			lastCheck = new Timestamp(new Date().getTime());
			
		} catch (SAXException e) {
			dbgerr(e);
		}
		// catch (IntegrationException e) {
		// dbgerr(e);
		// } catch (FactException e) {
		// dbgerr(e);
		// } catch (IOException e) {
		// dbgerr(e);
		// } catch (ClassNotFoundException e) {
		// dbgerr(e);
		// }
	}

	/*
	 * @Override (non-Javadoc)
	 *
	 * @see coms.component.Component#execute()
	 */
	public void execute() throws Exception {
	}

	/*
	 * @Override (non-Javadoc)
	 *
	 * @see coms.component.Component#unload()
	 */
	public void unload() {
	}

	/***************************************************************************
	 *
	 * Add and remove methods for frame, sea cage, sensor and sensor values
	 *
	 **************************************************************************/

	// private void addFrame(String location, String description, String mobile,
	private void addFrame(Frame f) {
		try {
			String frameID = "";

			// check if frame exists
			Select sel = new Select("frame", getMetaData().getDbType());
			sel.and("location", Select.EQUAL, f.getLocation());
			sel.and("description", Select.EQUAL, f.getDescription());
			sel.and("mobile", Select.EQUAL, f.getMobile());
			sel.and("email", Select.EQUAL, f.getEmail());
			sel.column("id");

			List<Map<String, Object>> list = sel.execute();

			Iterator it = list.iterator();
			while (it.hasNext()) {
				HashMap frame = (HashMap) it.next();
				frameID = String.valueOf((Integer) frame.get("id"));
			}

			// frame doesn't exists
			if (frameID.length() == 0) {
				// create the frame
				Insert in = new Insert("frame", getMetaData().getDbType());
				in.column("location", f.getLocation());
				in.column("description", f.getDescription());
				in.column("mobile", f.getMobile());
				in.column("email", f.getEmail());
				in.execute();
			}

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
	}

	private void updateFrame(Frame f) {

		try {

			Update update = new Update("frame", getMetaData().getDbType());
			update.column("location", f.getLocation());
			update.column("description", f.getDescription());
			update.column("mobile", f.getMobile());
			update.column("email", f.getEmail());
			update.column("updated", 1);
			update.and("id", Update.EQUAL, f.getId());
			update.execute();

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

		// return "<xml></xml>";
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see scg.InputXmlSCGMBean#removeFrame(java.lang.String)
	 */
	private boolean removeFrame(int id) {
		try {

			Select sel = new Select("seacage", getMetaData().getDbType());
			sel.and("frameid", Select.EQUAL, id);
			sel.column("id");

			List<Map<String, Object>> list = sel.execute();

			Iterator it = list.iterator();

			// need to manually remove all sea cages in connection with the
			// frame
			boolean recurseOK = true;
			while (it.hasNext() && recurseOK) {
				HashMap seacage = (HashMap) it.next();
				int seaCageId = (Integer) seacage.get("id");

				recurseOK = removeSeaCage(seaCageId);

			}

			// delete the frame
			Delete del = new Delete("frame", getMetaData().getDbType());
			del.and("id", Delete.EQUAL, id);
			del.execute();

			return true;

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

		return false;
	}

	private void addSeaCage(SeaCage sc) {

		try {
			String seacageID = "";

			// checking if a sea cage with the given imei number at the given
			// frame already exists
			Select sel = new Select("seacage", getMetaData().getDbType());
			sel.column("id");
			sel.and("frameid", Select.EQUAL, sc.getFrameId());
			sel.and("imei", Select.EQUAL, sc.getImei());

			List<Map<String, Object>> list = sel.execute();

			Iterator it = list.iterator();
			while (it.hasNext()) {
				HashMap seacage = (HashMap) it.next();
				Integer id = (Integer) seacage.get("id");
				seacageID = id.toString();
			}

			// create the sea cage
			if (seacageID.length() <= 0) {
				Insert in = new Insert("seacage", getMetaData().getDbType());

				// not really needed... fish farmes don't care about satellites
				// and altitude should be zero at all times =)
				// in.column("satellites", 0);
				// in.column("altitude", 0);

				// no GPS data set yet
				in.column("latitude", "No GPS");
				in.column("longitude", "No GPS");

				in.column("description", sc.getDescription());
				in.column("imei", sc.getImei());
				in.column("interval", sc.getInterval());
				in.column("frameid", sc.getFrameId());
				in.execute();
			}

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
	}

	private void updateSeaCage(SeaCage sc) {
		try {

			Update update = new Update("seacage", getMetaData().getDbType());
			update.column("imei", sc.getImei());
			update.column("description", sc.getDescription());
			update.column("interval", sc.getInterval());
			update.and("id", Update.EQUAL, sc.getId());
			update.execute();

			// need the frame id of the sea cage to set the updated flag
			Select sel = new Select("seacage", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, sc.getId());
			int frameid = (Integer) ((HashMap) sel.execute().iterator().next()).get("frameid");

			// set the updated flag
			update = new Update("frame", getMetaData().getDbType());
			update.column("updated", 1);
			update.and("id", Update.EQUAL, frameid);
			update.execute();

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
	}

	private void updateGPSForSeaCage(GPS gps) {
		try {
			dbg("updateGPSForSeaCage");
//			get the initial position
			Select select = new Select("seacage", getMetaData().getDbType());
			select.column("initiallatitude");
			select.column("initiallongitude");
			select.and("id", Select.EQUAL, gps.getSeacageID());

			List<Map<String, Object>> list = select.execute();
			Iterator it = list.iterator(); 
			HashMap iLatLng  = null;
			if(it.hasNext())
				 iLatLng = (HashMap) it.next();

			if( iLatLng != null &&
				!(iLatLng.get("initiallatitude") == null || iLatLng.get("initiallongitude") == null) &&
				!(iLatLng.get("initiallatitude") == "No GPS" || iLatLng.get("initiallongitude") == "No GPS")) {
//				get the object for the initial position
				LatLng initialLatLng = new LatLng(
						Double.parseDouble((String) iLatLng.get("initiallatitude")),
						Double.parseDouble((String) iLatLng.get("initiallongitude")));

//				dbg(initialLatLng.toString());
//				dbg(gps.getLatLng().toString());

//				measure distance between to posistions.. in km. therefore * 1000
				double distance = initialLatLng.distance(gps.getLatLng())  * 1000.0;

				if(distance > 50) {

					BigDecimal bd = new BigDecimal(distance);
					bd = bd.setScale(2, BigDecimal.ROUND_UP);
					
					distance = bd.doubleValue();
					
					dbg("out of position: " + distance + " > 50");
					
					Select selAlarm = new Select("alarm", getMetaData().getDbType());
					selAlarm.and("id", Select.EQUAL, gps.getSeacageID());
					
					if(selAlarm.execute().size() == 0) {
					
	//					generate alarm message
						String message = "Sea cage med ID " + gps.getSeacageID() +
							" er " + distance + "m ute av posisjon!";
	
	//					store it in the database
						Insert in = new Insert("alarm", getMetaData().getDbType());
						in.column("sensorid", -1);
						in.column("seacageid", gps.getSeacageID());
						in.column("message", message);
						in.column("issent", 0);
						in.column("confirmed", new Timestamp(0));
	
						in.execute();
					}
				} else {
					dbg("Sea cage is in position (distance: " + distance + ")");
				}
			} else {
				dbg("GPS not set yet");
				Update update = new Update("seacage", getMetaData().getDbType());
				update.column("initiallatitude", (gps.getLatLng() == null ? "No GPS" : gps.getLatLng().getLat()));
				update.column("initiallongitude", (gps.getLatLng() == null ? "No GPS" : gps.getLatLng().getLng()));

				update.and("id", Update.EQUAL, gps.getSeacageID());
				update.execute();
			}

			// update sea cage gps data
			Update update = new Update("seacage", getMetaData().getDbType());
			update.column("latitude", (gps.getLatLng() == null ? "No GPS" : gps.getLatLng().getLat()));
			update.column("longitude", (gps.getLatLng() == null ? "No GPS" : gps.getLatLng().getLng()));

			update.and("id", Update.EQUAL, gps.getSeacageID());
			update.execute();


		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		} catch (Exception e) {
			dbgerr(e);
		}
	}

	private boolean removeSeaCage(int seaCageId) {
		try {

			Select sel = new Select("sensor", getMetaData().getDbType());
			sel.and("seacageid", Select.EQUAL, seaCageId);
			sel.column("id");

			List<Map<String, Object>> list = sel.execute();

			Iterator it = list.iterator();

			// need to remove all sensors connected to the sea cage
			boolean recurseOK = true;
			while (it.hasNext() && recurseOK) {
				HashMap sensor = (HashMap) it.next();
				int sensorid = (Integer) sensor.get("id");

				recurseOK = removeSensor(sensorid);

			}

			sel = new Select("seacage", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, seaCageId);
			int frameid = (Integer) ((HashMap) sel.execute().iterator().next())
					.get("frameid");

			Update update = new Update("frame", getMetaData().getDbType());
			update.column("updated", 1);
			update.and("id", Update.EQUAL, frameid);
			update.execute();

			// delete the sea cage
			Delete del = new Delete("seacage", getMetaData().getDbType());
			del.and("id", Delete.EQUAL, seaCageId);
			del.execute();

			return true;
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

		return false;
	}

	private void addSensor(Sensor s) {

		try {
			String sensorID = "";
			// check if the sensor exists
			Select sel = new Select("sensor", getMetaData().getDbType());
			sel.column("id");
			sel.and("sensortypeid", Select.EQUAL, s.getSensorTypeId());
			sel.and("seacageid", Select.EQUAL, s.getSeaCageID());

			List<Map<String, Object>> list = sel.execute();

			Iterator it = list.iterator();
			while (it.hasNext()) {
				HashMap sensor = (HashMap) it.next();
				Integer id = (Integer) sensor.get("id");
				sensorID = id.toString();
			}

			// sensor does not exist
			if (sensorID.length() <= 0) {
				Insert in = new Insert("sensor", getMetaData().getDbType());
				in.column("sensortypeid", s.getSensorTypeId());

				// the range of possible sensor values
				// in.column("maxrange", maxrange);
				// in.column("minrange", minrange);

				// the range of legal sensor values
				in.column("maxvalue", s.getMaxValue());
				in.column("minvalue", s.getMinValue());

				in.column("seacageid", s.getSeaCageID());
				in.execute();
			}
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
	}

	private void updateSensor(Sensor s) {
		try {

			Update update = new Update("sensor", getMetaData().getDbType());
			update.column("sensortypeid", s.getSensorTypeId());
			update.column("maxvalue", s.getMaxValue());
			update.column("minvalue", s.getMinValue());
			update.column("seacageid", s.getSeaCageID());
			update.and("id", Update.EQUAL, s.getId());
			update.execute();

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

	}

	private boolean removeSensor(int id) {
		try {

			Select sel = new Select("sensor", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, id);
			int seaCageId = (Integer) sel.execute().iterator().next().get(
					"seacageid");

			sel = new Select("seacage", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, seaCageId);
			int frameid = (Integer) ((HashMap) sel.execute().iterator().next())
					.get("frameid");

			Update update = new Update("frame", getMetaData().getDbType());
			update.column("updated", 1);
			update.and("id", Update.EQUAL, frameid);
			update.execute();

			// deleting all sensor values for the given sensor
			Delete del = new Delete("sensorvalue", getMetaData().getDbType());
			del.and("sensorid", Delete.EQUAL, id);
			del.execute();

			// deleteing the sensor
			del = new Delete("sensor", getMetaData().getDbType());
			del.and("id", Delete.EQUAL, id);
			del.execute();

			return true;
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see scg.InputXmlSCGMBean#addSensorValue(java.lang.String,
	 *      java.lang.String)
	 */
	private void addSensorValue(SensorValue sv) {
		try {
			dbg("addSensorValue");
			// select the sensor values sesor to get allowed max + min values
			Select sel = new Select("sensor", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, sv.getSensorID());

			List<Map<String, Object>> list = sel.execute();

			Sensor sensor = new Sensor((HashMap) list.iterator().next());

			// select the sensor's sensor type to get the sensor value range
			// (for converting)
			sel = new Select("sensortype", getMetaData().getDbType());
			sel.and("id", Select.EQUAL, sensor.getSensorTypeId());

			SensorType sensorType = new SensorType((HashMap) sel.execute().iterator().next());

			// convert raw sensor data (millivolts) to real values
			double realSensorValue =
				Convert.convert(
						sv.getValue(),
						sensorType.getMaxRange(),
						sensorType.getMinRange());

			BigDecimal bd = new BigDecimal(realSensorValue);
			bd = bd.setScale(2, BigDecimal.ROUND_UP);
			
			realSensorValue = bd.doubleValue();
			
			// check if the sensor value is out fo bounds
			outofbounds = false;
			if (realSensorValue > sensor.getMaxValue()||
				realSensorValue < sensor.getMinValue())
				outofbounds = true;

			Timestamp now = new Timestamp(new Date().getTime());
			// add the sensor value, don't care 'bout returning id..
			Insert in = new Insert("sensorvalue", getMetaData().getDbType());
			in.column("value", realSensorValue);
			in.column("sensorid", sv.getSensorID());
			in.column("received", now);
			in.execute();

			if (outofbounds) {
				sel = new Select("alarm", getMetaData().getDbType());
				sel.and("sensorid", Select.EQUAL, sv.getSensorID());
				// sel.and("isconfirmed", Select.EQUAL, 0);
				sel.and("confirmed", Select.EQUAL, new Timestamp(0));
				sel.and("issent", Select.EQUAL, 1);

				if (sel.execute().size() == 0) {

					sel = new Select("sensorvalue", getMetaData().getDbType());
					sel.and("received", Select.EQUAL, now);
					sel.and("sensorid", Select.EQUAL, sv.getSensorID());

					SensorValue sv1 =
						new SensorValue((HashMap) sel.execute().iterator().next());

//					generate a message for the alarm
					String message = "Sea cage med ID " + sensor.getSeaCageID() +
					" sin " + sensorType.getType() + "sensor er utenfor tillatte verdier" +
					" (" + sensor.getMaxValue() + "/" + sensor.getMinValue() + ")" +
					" med verdien " + sv1.getValue() + "" + sensorType.getUnit() + "." +
					" Verdien ble mottatt " + sv1.getReceived().toLocaleString();
					
//					store it in the database
					in = new Insert("alarm", getMetaData().getDbType());
					in.column("sensorid", sv1.getSensorID());
					in.column("seacageid", sensor.getSeaCageID());
					in.column("message", message);
					in.column("issent", 0);
					in.column("confirmed", new Timestamp(0));

					in.execute();

					dbg("added an alarm.. ");
				} else
					dbg("alarm for sensor allready exists");
			}
		
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		} catch (Exception e) {
			dbgerr(e);
		}
	}

	private String addSensorType(SensorType st) {
		try {
			Insert in = new Insert("sensortype", getMetaData().getDbType());
			in.column("unit", st.getUnit());
			in.column("type", st.getType());
			in.column("description", st.getDescription());
			in.column("maxrange", st.getMaxRange());
			in.column("minrange", st.getMinRange());
			in.execute();
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

		return "";
	}

	// private String addSensorType(String type, String description,
	// String maxrange, String minrange) {
	// try {
	// Insert in = new Insert("sensortype", getMetaData().getDbType());
	// in.column("type", type);
	// in.column("description", description);
	// in.column("maxrange", maxrange);
	// in.column("minrange", minrange);
	// in.execute();
	// } catch (SQLException e) {
	// dbgerr(e);
	// } catch (ConnectionException e) {
	// dbgerr(e);
	// }
	//
	// return "";
	// }
	//
	// private String updateSensorType(String id, String type, String
	// description,
	// String maxrange, String minrange) {
	// try {
	// Update update = new Update("sensortype", getMetaData().getDbType());
	// update.column("type", type);
	// update.column("description", description);
	// update.column("maxrange", maxrange);
	// update.column("minrange", minrange);
	// update.and("id", id);
	// update.execute();
	//
	// } catch (SQLException e) {
	// dbgerr(e);
	// } catch (ConnectionException e) {
	// dbgerr(e);
	// }
	// return "";
	// }
	//
	// private String removeSensorType(String id) {
	// try {
	// Delete delete = new Delete("sensortype", getMetaData().getDbType());
	// delete.and("id", Delete.EQUAL, id);
	// delete.execute();
	// } catch (SQLException e) {
	// dbgerr(e);
	// } catch (ConnectionException e) {
	// dbgerr(e);
	// }
	// return "";
	// }

	/*
	 * (non-Javadoc)
	 *
	 * @see com.telenor.apms.fifamos.InputMBean#parseXml(java.lang.String)
	 */
	public String postSourceXml(String xml) {
		try {
			dbg("Source XML received....");
			dbg(xml);
			parser.parse(
					new InputSource(
							new ByteArrayInputStream(xml.getBytes())));

		} catch (IOException e) {
			dbgerr(e);
		} catch (SAXException e) {
			dbgerr(e);
		}

		return generateXml();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.telenor.apms.fifamos.InputMBean#postConsumerXml(java.lang.String)
	 */
	public String postConsumerXml(String xml) {
		try {
			dbg("Consumer XML received....");
			dbg(xml);
			parser.parse(
					new InputSource(
							new ByteArrayInputStream(xml.getBytes())));

		} catch (IOException e) {
			dbgerr(e);
		} catch (SAXException e) {
			dbgerr(e);
		}

		return "<xml></xml>";
	}

	/**
	 * Generate xml with commands for the m2m-module.
	 *
	 * @return
	 */
	private String generateXml() {
		dbg("generate new xml");
		StringBuffer xml = new StringBuffer();
		xml.append("<xml>");
		try {

			Select selAlarm = new Select("alarm", getMetaData().getDbType());
			selAlarm.and("issent", Select.EQUAL, 0);

			List<Map<String, Object>> listAlarm = selAlarm.execute();

			Iterator it = listAlarm.iterator();

			while (it.hasNext()) {
				HashMap alarm = (HashMap) it.next();

				xml.append("<alarm id=\"" + alarm.get("id") + "\">\n");
				xml.append("<seacageid>" + alarm.get("seacageid") + "</seacageid>\n");
				xml.append("<message>" + alarm.get("message") + "</message>\n");
				xml.append("</alarm>");

				Update update = new Update("alarm", getMetaData().getDbType());
				update.column("issent", 1);
				update.and("id", Update.EQUAL, alarm.get("id"));
				update.execute();
			}

			// select frames that are updated or has sea cages/sensors that are
			// updated..
			Select selFrame = new Select("frame", getMetaData().getDbType());
			selFrame.and("updated", Select.EQUAL, 1);

			List<Map<String, Object>> listUpdatedSeaCage = selFrame.execute();

			if(listUpdatedSeaCage.size()>0) {
				xml.append("<config>\n");
				Iterator it2 = listUpdatedSeaCage.iterator();
				while (it2.hasNext()) {
					HashMap frame = (HashMap) it2.next();

					xml.append("<updated frameid=\"" + frame.get("id") + "\"/>\n");

					Update update = new Update("frame", getMetaData().getDbType());
					update.column("updated", 0);
					update.and("id", Update.EQUAL, frame.get("id"));
					update.execute();
				}

				xml.append("</config>\n");
			}

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}

		xml.append("</xml>");
		dbg(xml.toString());
		return xml.toString();

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.telenor.apms.fifamos.InputMBean#confirmAlarms()
	 */
	public String confirmAlarms() {

		try {
			Update update = new Update("alarm", getMetaData().getDbType());
			update.column("confirmed", new Timestamp(new Date().getTime()));
			update.and("confirmed", Update.EQUAL, new Timestamp(0));

			update.execute();

			return "ok";
		} catch (SQLException e) {
			dbgerr(e);
			return e.getMessage();
		} catch (ConnectionException e) {
			dbgerr(e);
			return e.getMessage();
		}

	}

	/*
	 * (non-Javadoc)
	 *
	 * @see com.telenor.apms.fifamos.XmlListener#passResult(java.lang.Object)
	 */
	public void passResult(Object obj) {
		try {
			if (obj instanceof Command) {
				Command command = (Command) obj;
				if (command.getType().equals("add")) {
					if (command.getObj() instanceof Frame) {
						Frame frame = (Frame) command.getObj();
						addFrame(frame);

					} else if (command.getObj() instanceof SeaCage) {
						SeaCage seacage = (SeaCage) command.getObj();
						addSeaCage(seacage);

					} else if (command.getObj() instanceof Sensor) {
						Sensor sensor = (Sensor) command.getObj();
						addSensor(sensor);

					} else if (command.getObj() instanceof SensorValue) {
						SensorValue sv = (SensorValue) command.getObj();
						addSensorValue(sv);

					}
				} else if (command.getType().equals("update")) {
					if (command.getObj() instanceof Frame) {
						Frame frame = (Frame) command.getObj();
						updateFrame(frame);

					} else if (command.getObj() instanceof SeaCage) {
						SeaCage seacage = (SeaCage) command.getObj();
						updateSeaCage(seacage);

					} else if (command.getObj() instanceof Sensor) {
						Sensor sensor = (Sensor) command.getObj();
						updateSensor(sensor);
					} else if (command.getObj() instanceof GPS) {
//						System.out.println("got GPS DATA !!! ");
						GPS gps = (GPS) command.getObj();
						updateGPSForSeaCage(gps);
					}
				} else if (command.getType().equals("remove")) {
					if (command.getObj() instanceof Frame) {
						Frame frame = (Frame) command.getObj();
						removeFrame(frame.getId());

					} else if (command.getObj() instanceof SeaCage) {
						SeaCage seacage = (SeaCage) command.getObj();
						removeSeaCage(seacage.getId());

					} else if (command.getObj() instanceof Sensor) {
						Sensor sensor = (Sensor) command.getObj();
						removeSensor(sensor.getId());
					}
				}
			}
		} catch (Exception e) {
			dbgerr(e);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see scg.InputXmlSCGMBean#cleanDB()
	 */
	public String cleanDB() {
		try {
			// quick way of restarting the database, too bad the indexes isn't
			// reset..

			Delete del = new Delete("sensorvalue", getMetaData().getDbType());
			del.execute();

			del = new Delete("sensor", getMetaData().getDbType());
			del.execute();

			del = new Delete("sensortype", getMetaData().getDbType());
			del.execute();

			del = new Delete("seacage", getMetaData().getDbType());
			del.execute();

			del = new Delete("frame", getMetaData().getDbType());
			del.execute();

			del = new Delete("alarm", getMetaData().getDbType());
			del.execute();

			dbg("database had been cleaned...");

			return "";
		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		}
		return "done";
	}

	public String fillDB(String frames, String seaCages) {
		try {
			cleanDB();

			SensorType st1 = new SensorType(0, 100, 0, "%", "Oksygen",
					"Maaler oksygennivaa");
			SensorType st2 = new SensorType(0, 14, 0, "pH", "Surhet",
					"Maaler pH-verdi");
			SensorType st3 = new SensorType(0, 100, 0, "%",
					"Gjennomstroemming", "Maaler gjennomstroemming av vann");
			SensorType st4 = new SensorType(0, 40000, 0, "lux", "Lysforhold",
					"Maaler lysforhold");
			SensorType st5 = new SensorType(0, 120, -40, "C", "Temperatur",
					"Maaler temperaturen");
			SensorType st6 = new SensorType(0, 100, 0, "%", "Matnivaa",
					"Maaler matnivaaet");

			addSensorType(st1);
			addSensorType(st5);
			addSensorType(st6);
			addSensorType(st3);
			addSensorType(st2);
			addSensorType(st4);

			int numOfFrames = Integer.parseInt(frames);
			int numOfSeaCages = Integer.parseInt(seaCages);

			int justAnotherImei = 0;

			for (int i = 0; i < numOfFrames; i++) {
				Insert insertFrame = new Insert("frame",
						getMetaData().getDbType());
				insertFrame.column("location", "Frame #" + i);
				insertFrame.column("description", "Telenor test cage");
				insertFrame.column("mobile", "91808620");
				insertFrame.column("email", "jagrodal@broadpark.no");
				insertFrame.execute();

				Select selectFrame = new Select("frame",
						getMetaData().getDbType());
				selectFrame.and("location", Select.EQUAL, "Frame #" + i);

				List<Map<String, Object>> listFrame = selectFrame.execute();

				Iterator itFrame = listFrame.iterator();

				HashMap frame = (HashMap) itFrame.next();
				int frameid = (Integer) frame.get("id");

				for (int j = 0; j < numOfSeaCages; j++) {
					Insert insertSeaCage = new Insert("seacage",
							getMetaData().getDbType());

					// no GPS data set yet
					insertSeaCage.column("latitude", "No GPS");
					insertSeaCage.column("longitude", "No GPS");

					String description = "Sea cage #" + i + "-" + j;
					if (i == 0 && j == 0)
						description += " (Aplicom 12)";

					insertSeaCage.column("description", description);

					if (i == 0 && j == 0)
						// insertSeaCage.column("imei", "352540000049568");
						insertSeaCage.column("imei", "352540000046895");
					else
						insertSeaCage.column("imei", justAnotherImei++);
					insertSeaCage.column("interval", 30);
					insertSeaCage.column("frameid", frameid);
					insertSeaCage.execute();

					// getting the id of the newly created sea cage
					Select selectSeaCage = new Select("seacage", getMetaData().getDbType());
					selectSeaCage.and("description", Select.EQUAL, description);
					List<Map<String, Object>> listSeaCage = selectSeaCage.execute();

					Iterator itSeaCage = listSeaCage.iterator();
					HashMap seacage = (HashMap) itSeaCage.next();
					int seacageid = (Integer) seacage.get("id");

					// getting all sensor types and their values
					Select selSensorType = new Select("sensortype", getMetaData().getDbType());
					List<Map<String, Object>> listSensorType = selSensorType.execute();
					
					int sensorTypeSize = listSensorType.size();
					dbg("SensorType List size: " + sensorTypeSize);

					Iterator itSensorType = listSensorType.iterator();
					itSensorType.hasNext();

					if (i == 0 && j == 0)
						sensorTypeSize = 3;

					for (int k = 0; k < sensorTypeSize; k++) {

						HashMap sensorType = (HashMap) itSensorType.next();
						Insert insertSensor = new Insert("sensor", getMetaData().getDbType());
						insertSensor.column("sensortypeid", sensorType.get("id"));

						// the range of legal sensor values
						double maxrange = (Double) sensorType.get("maxrange");
						maxrange = maxrange * 0.7;
						BigDecimal bd = new BigDecimal(maxrange);
						bd = bd.setScale(2, BigDecimal.ROUND_UP);
						
						maxrange = bd.doubleValue();
						
						insertSensor.column("maxvalue", maxrange);
						double minrange = (Double) sensorType.get("minrange");
						minrange = ((maxrange - Math.abs(minrange)) * 0.3);
						
						bd = new BigDecimal(minrange);
						bd = bd.setScale(2, BigDecimal.ROUND_UP);
						
						minrange = bd.doubleValue();
						
						insertSensor.column("minvalue", minrange);

						insertSensor.column("seacageid", seacageid);
						insertSensor.execute();

						dbg("ferdig med sensor #" + k + " p sea cage #" + j + " p frame #" + i);
					}

					dbg("ferdig med sea cage #" + j);
				}
				dbg("ferdig med frame #" + i);
			}
			dbg("All done");

		} catch (SQLException e) {
			dbgerr(e);
		} catch (ConnectionException e) {
			dbgerr(e);
		} catch (Exception e) {
			dbgerr(e);
		}

		return "";

	}

	private void dbg(String msg) {
		System.out.println(msg);
	}

	private void dbgerr(Exception err) {
		err.printStackTrace();
		err.getCause();
	}
}
