/*****************************************************************************
 * 																			 *
 * This document is formated with a width of 80 characters. 				 *
 * Please keep it that way. 												 *
 * 																			 *
 *****************************************************************************/
package com.telenor.apms.fifamos;

import coms.component.ComponentMBean;

/**
 * @author Frank
 * 
 */
public interface OutputMBean extends ComponentMBean {

	
	/**
	 * Method for getting all frames in the system
	 * 
	 * @param location
	 *            the location of the frame
	 * @param description
	 *            the description of the frame
	 * @param mobile
	 *            the mobile of the responsible of the frame
	 * @param email
	 *            the email of the responsible of the frame
	 * @return a character separated String representation of all the frames or
	 *         the ones which match the mobilenumber if provided this will be a
	 *         XML representation soon
	 */
	public String getFrames(String location, String description, String mobile,
			String email);

	/**
	 * Method for getting all sea cages for a given frame
	 * 
	 * @param frameid
	 *            the id of the framein question
	 * @return a character separated String representation of alle the sea cages
	 *         this will be a XML representation soon
	 */
	public String getSeaCagesForFrame(String frameid);

	/**
	 * Method for getting all sensors for a given sea cage
	 * 
	 * @param seacageid
	 *            the id of the sea cage in question
	 * @return a character separated String representation of all the sensors
	 *         this will be a XML representation soon
	 */
	public String getSensorsForSeaCage(String seacageid);

	/**
	 * Method for getting all sensor values for a given sensor during a given
	 * period of time
	 * 
	 * @param sensorid
	 *            the id of the sensor in question
	 * @param period
	 *            the period of time given in seconds (60*60*24 would give the
	 *            values for the last 24 hours)
	 * 
	 * @return a character separated String representation of alle the frames
	 *         this will be a XML representation soon
	 */
	public String getHistoryForSensor(String sensorid, String period);

	/**
	 * Method for initializing the m2m-device. This method will give the
	 * m2m-device information on what frame and sea cage it belongs to, and
	 * which sensors it has
	 * 
	 * @param imei
	 *            the unique imei number of the m2m-device in question
	 * @return a XML-representation of the device info
	 */
	public String getConfigAsXml(String imei);
	

	/**
	 * Method for initializing the client application. This method will give the
	 * application information on which frames, sea cages and
	 * sensors the fish farm has
	 * 
	 * @return a XML-representation of the fish farm
	 */
	public String getFishFarmAsXml();

	/**
	 * Method for getting outstanding alarms
	 * 
	 * @return a String representation of all the alarms
	 */
	public String getAlarms();

	/**
	 * Method for getting outstanding alarms
	 * 
	 * @return a String representation of all the alarms
	 */
	public String getGpsData(String seacageids);
	
	/**
	 * @return the sensor types
	 */
	public String getSensorTypes();

}
