package com.telenor.apms.fifamos.utils;

import java.util.Stack;
import java.util.StringTokenizer;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import uk.me.jstott.jcoord.LatLng;

import com.telenor.apms.fifamos.objects.Command;
import com.telenor.apms.fifamos.objects.Frame;
import com.telenor.apms.fifamos.objects.GPS;
import com.telenor.apms.fifamos.objects.SeaCage;
import com.telenor.apms.fifamos.objects.Sensor;
import com.telenor.apms.fifamos.objects.SensorType;
import com.telenor.apms.fifamos.objects.SensorValue;

/**
 * Parser handler class to parse the XML input stream.
 * 
 * @author Frank
 * 
 */
public class XMLEventHandler extends DefaultHandler {

	// Private members needed to parse the XML document'
	// keep track of parsing
	private boolean parsingInProgress;

	// keep track of qName
	private Stack<String> qNameStack = new Stack<String>();

	private XMLEventListener listener;

	private GPS gps;

	private StringBuffer sb;

	private Command command;

	private Frame frame;

	private SeaCage seacage;

	private Sensor sensor;

	private SensorValue sensorvalue;

	private SensorType sensortype;

	private LatLng latlng;

	public XMLEventHandler(XMLEventListener listener) {
		this.listener = listener;
	}

	/**
	 * Set a listener to pass results from the xml parsing
	 * 
	 * @param listener
	 *            is the listener to use
	 */
	public void setListener(XMLEventListener listener) {
		this.listener = listener;
	}

	/**
	 * Remove the listener from the handler..
	 */
	public void removeListener() {
		this.listener = null;
	}

	/**
	 * Start of document processing.
	 * 
	 * @throws org.xml.sax.SAXException
	 *             is any SAX exception, possibly wrapping another exception.
	 */
	public void startDocument() throws SAXException {
		parsingInProgress = true;
		qNameStack.removeAllElements();
	}

	/**
	 * Process the new element.
	 * 
	 * @param uri
	 *            is the Namespace URI, or the empty if the element has no
	 *            Namespace URI or if Namespace processing is not being
	 *            performed.
	 * @param localName
	 *            is the The local name (without prefix), or the empty if
	 *            Namespace processing is not being performed.
	 * @param qName
	 *            is the qualified name (with prefix), or the empty if qualified
	 *            names are not available.
	 * @param attributes
	 *            is the attributes attached to the element. If there are no
	 *            attributes, it shall be an empty Attributes object.
	 * @throws org.xml.sax.SAXException
	 *             is any SAX exception, possibly wrapping another exception.
	 */
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		sb = new StringBuffer();
//		String attribs = "";
//		for (int i = 0; i < attributes.getLength(); i++) {
//			attribs += attributes.getValue(i) + " ";
//		}
//		dbg(qName + " - " + attribs);

		if (qName.equals("xml")) {

		} else if (qName.equals("command")) {
			command = new Command();
			command.setType(attributes.getValue("type"));
		} else if (qName.equals("frame")) {
			frame = new Frame();
		} else if (qName.equals("seacage")) {
			seacage = new SeaCage();
		} else if (qName.equals("sensor")) {
			sensor = new Sensor();
		} else if (qName.equals("sensortype")) {
			sensortype = new SensorType();
		} else if (qName.equals("sensorvalue")) {
			sensorvalue = new SensorValue();
		} else if (qName.equals("gps")) {
			gps = new GPS();
//		} else if (qName.equals("latitude")) {
//			latitude = new Latitude();
//		} else if (qName.equals("longitude")) {
//			longitude = new Longitude();
		} else if (qName.equals("latlng")) {

			double latdeg, latmin, latsec, lngdeg, lngmin, lngsec = 0;
			StringTokenizer latTk = new StringTokenizer(attributes.getValue("lat"), " ");
			latdeg = Double.parseDouble(latTk.nextToken());
			latmin = Double.parseDouble(latTk.nextToken());
			latsec = Double.parseDouble(latTk.nextToken());

			StringTokenizer lngTk = new StringTokenizer(attributes.getValue("lng"), " ");
			lngdeg = Double.parseDouble(lngTk.nextToken());
			lngmin = Double.parseDouble(lngTk.nextToken());
			lngsec = Double.parseDouble(lngTk.nextToken());

			latlng = new LatLng(
					latdeg + latmin/60.0 + latsec/3600.0, 
					lngdeg + lngmin/60.0 + lngsec/3600.0);
			
		} else if (qName.equals("id")) {
		} else if (qName.equals("maxrange")) {
		} else if (qName.equals("minrange")) {
		} else if (qName.equals("maxvalue")) {
		} else if (qName.equals("minvalue")) {
		} else if (qName.equals("sensorid")) {
		} else if (qName.equals("value")) {
		} else if (qName.equals("seacageid")) {
		} else if (qName.equals("satellites")) {
		} else if (qName.equals("altitude")) {
		} else if (qName.equals("degrees")) {
		} else if (qName.equals("minutes")) {
		} else if (qName.equals("seconds")) {
		} else if (qName.equals("milliseconds")) {
		} else if (qName.equals("type")) {
		} else if (qName.equals("description")) {
		} else if (qName.equals("mobile")) {
		} else if (qName.equals("email")) {
		} else if (qName.equals("interval")) {
		} else if (qName.equals("imei")) {
		} else if (qName.equals("location")) {
		}

		// Keep track of QNames
		qNameStack.push(qName);
	}

	/**
	 * Process the character data for current tag.
	 * 
	 * @param ch
	 *            are the element's characters.
	 * @param start
	 *            is the start position in the character array.
	 * @param length
	 *            is the number of characters to use from the character array.
	 * @throws org.xml.sax.SAXException
	 *             is any SAX exception, possibly wrapping another exception.
	 */
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		String s = new String(ch, start, length).trim();
		sb.append(s.trim());

	}

	/**
	 * Process the end element tag.
	 * 
	 * @param uri
	 *            is the Namespace URI, or the empty if the element has no
	 *            Namespace URI or if Namespace processing is not being
	 *            performed.
	 * @param localName
	 *            is the The local name (without prefix), or the empty if
	 *            Namespace processing is not being performed.
	 * @param qName
	 *            is the qualified name (with prefix), or the empty if qualified
	 *            names are not available.
	 * @throws org.xml.sax.SAXException
	 *             is any SAX exception, possibly wrapping another exception.
	 */
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		// Pop QName, since we are done with it
		if(qNameStack.peek()!=null && qNameStack.peek()!="")
			qNameStack.pop();

		// processing elements
		if (qName.equals("xml")) {
		} else if (qName.equals("command")) {
//			dbg("listener.passResult - " + command.getType() + " - " + command.getObj().toString());
			listener.passResult(command);
			command = null;
		} else if (qName.equals("frame")) {
			if (command != null)
				command.setObj(frame);
		} else if (qName.equals("seacage")) {
			if (command != null)
				command.setObj(seacage);
		} else if (qName.equals("sensor")) {
			if (command != null)
				command.setObj(sensor);
		} else if (qName.equals("sensortype")) {
			if (command != null)
				command.setObj(sensortype);
		} else if (qName.equals("sensorvalue")) {
			if (command != null)
				command.setObj(sensorvalue);
		} else if (qName.equals("gps")) {
			if (command != null)
				command.setObj(gps);
//		} else if (qName.equals("latitude")) {
//			if (qNameStack.peek().equals("gps")) {
////				gps.setLatitude(latitude);
//			} else if (qNameStack.peek().equals("seacage")) {
//				if (sb.toString().length() == 1)
//					seacage.setLatitude("No GPS");
//				else
//					seacage.setLatitude(sb.toString());
//			}
//		} else if (qName.equals("longitude")) {
//			if (qNameStack.peek().equals("gps")) {
////				gps.setLongitude(longitude);
//			} else if (qNameStack.peek().equals("seacage")) {
//				if (sb.toString().length() == 0)
//					seacage.setLongitude("No GPS");
//				else
//					seacage.setLongitude(sb.toString());
//			}
		} else if (qName.equals("latlng")) {
			gps.setLatLng(latlng);
		} else if (qName.equals("id")) {
			if (qNameStack.peek().equals("sensor")) {
				sensor.setId(Integer.parseInt(sb.toString()));
			} else if (qNameStack.peek().equals("frame")) {
				frame.setId(Integer.parseInt(sb.toString()));
			} else if (qNameStack.peek().equals("seacage")) {
				seacage.setId(Integer.parseInt(sb.toString()));
			}
		} else if (qName.equals("maxrange")) {
			if (qNameStack.peek().equals("sensortype")) {
				sensortype.setMaxRange(Double.parseDouble(sb.toString()));
			}
		} else if (qName.equals("minrange")) {
			if (qNameStack.peek().equals("sensortype")) {
				sensortype.setMinRange(Double.parseDouble(sb.toString()));
			}
		} else if (qName.equals("sensortypeid")) {
			sensor.setSensorTypeId(Integer.parseInt(sb.toString()));
		} else if (qName.equals("maxvalue")) {
			if (qNameStack.peek().equals("sensor")) {
				sensor.setMaxValue(Double.parseDouble(sb.toString()));
			}
		} else if (qName.equals("minvalue")) {
			if (qNameStack.peek().equals("sensor")) {
				sensor.setMinValue(Double.parseDouble(sb.toString()));
			}
		} else if (qName.equals("sensorid")) {
			if (qNameStack.peek().equals("sensorvalue")) {
				sensorvalue.setSensorID(Integer.parseInt(sb.toString()));
			} /*else if (qNameStack.peek().equals("sensor")) {
				sensor.setId(Integer.parseInt(sb.toString()));
			}*/
		} else if (qName.equals("value")) {
			if (qNameStack.peek().equals("sensorvalue")) {
				sensorvalue.setValue(Double.parseDouble(sb.toString()));
			}
		} else if (qName.equals("seacageid")) {
			if(gps!=null) {
				gps.setSeacageID(Integer.parseInt(sb.toString()));
			}
			if (qNameStack.peek().equals("sensor")) {
				sensor.setSeaCageID(Integer.parseInt(sb.toString()));
			}
		} else if (qName.equals("satellites")) {
			gps.setSatellites(Integer.parseInt(sb.toString()));
		} else if (qName.equals("altitude")) {
			gps.setAltitude(Integer.parseInt(sb.toString()));
		} else if (qName.equals("interval")) {
			seacage.setInterval(Integer.parseInt(sb.toString()));
		} else if (qName.equals("frameid")) {
			seacage.setFrameId(Integer.parseInt(sb.toString()));
		} else if (qName.equals("imei")) {
			seacage.setImei(sb.toString());
//		} else if (qName.equals("degrees")) {
//			if (qNameStack.peek().equals("latitude")) {
//				latitude.setDegrees(sb.toString());
//			} else {
//				longitude.setDegrees(sb.toString());
//			}
//		} else if (qName.equals("minutes")) {
//			if (qNameStack.peek().equals("latitude")) {
//				latitude.setMinutes(sb.toString());
//			} else {
//				longitude.setMinutes(sb.toString());
//			}
//		} else if (qName.equals("seconds")) {
//			if (qNameStack.peek().equals("latitude")) {
//				latitude.setSeconds(sb.toString());
//			} else {
//				longitude.setSeconds(sb.toString());
//			}
//		} else if (qName.equals("milliseconds")) {
//			if (qNameStack.peek().equals("latitude")) {
//				latitude.setMilliseconds(sb.toString());
//			} else {
//				longitude.setMilliseconds(sb.toString());
//			}
//		} else if (qName.equals("type")) {
//			if (qNameStack.peek().equals("latitude")) {
//				latitude.setType(sb.toString());
//			} else if (qNameStack.peek().equals("longitude")) {
//				longitude.setType(sb.toString());
//			} else if (qNameStack.peek().equals("sensortype")) {
//				sensortype.setType(sb.toString());
//			}
		} else if (qName.equals("description")) {
			if (qNameStack.peek().equals("sensortype")) {
				sensortype.setDescription(sb.toString());
			} else if (qNameStack.peek().equals("frame")) {
				frame.setDescription(sb.toString());
			} else if (qNameStack.peek().equals("seacage")) {
				seacage.setDescription(sb.toString());
			}
		} else if (qName.equals("location")) {
			frame.setLocation(sb.toString());
		} else if (qName.equals("mobile")) {
			frame.setMobile(sb.toString());
		} else if (qName.equals("email")) {
			frame.setEmail(sb.toString());
		}
	}

	/**
	 * End of document processing.
	 * 
	 * @throws org.xml.sax.SAXException
	 *             is any SAX exception, possibly wrapping another exception.
	 */
	public void endDocument() throws SAXException {
		parsingInProgress = false;
	}

	private void dbg(String string) {
		 System.out.println("XmlEventHandler: " + string);
	}
}