package com.telenor.apms.fifamos.n12.logger;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Calendar;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

public class SerialPortLogger implements Log {

	private OutputStream outStream = null;
	private static SerialPortLogger instance;
	private StreamConnection sc = null;

	/**
	 * Initialize the serial port
	 */
	private SerialPortLogger() {

		try {

			sc = (StreamConnection) Connector.open("comm:3;baudrate=115200", Connector.READ_WRITE);

			// Open outputStstream
			outStream = sc.openOutputStream();
		} catch (IOException ignored) {

		}
	}

	/**
	 * Write message to the serial port.
	 * 
	 */
	public void write(String msg) {
		if (outStream != null && msg != null) {

			// prepare timestamp
			Calendar cal = Calendar.getInstance();

			// Print timestamp and message
			msg = "<[" + cal + "] " + msg + ">\n";

			try {

				outStream.write(msg.getBytes());

			} catch (IOException ignored) {
			}
		}
	}

	/**
	 * Returns instance of SerialPortLogger
	 * @return
	 */
	public static synchronized Log getInstance() {
		if (instance == null) {
			instance = new SerialPortLogger();
		}
		return instance;
	}

	/**
	 * Close the log. Release port resources.
	 */
	public void close() {

		if (outStream != null) {
			try {
				outStream.close();
			} catch (IOException ignored) {
			}
		}

		if (sc != null) {
			try {

				sc.close();
			} catch (IOException ignored) {
			}
		}
	}

}
