package com.telenor.apms.fifamos.n12.utils;

import com.nokia.m2m.orb.idl.gps.GpsData;
import com.nokia.m2m.orb.idl.gps.GpsModule;
import com.nokia.m2m.orb.idl.gps.GpsModuleHelper;
import com.nokia.m2m.orb.idl.gps.NoGpsDataAvailableException;

public class GPSControl {
	
	private GpsModule gpsModule;
	private GpsData data;

	private long latdeg;
	private long latmin;
	private long latsec;
	private long latmil;

	private long lngdeg;
	private long lngmin;
	private long lngsec;
	private long lngmil;

	public GPSControl(org.omg.CORBA.ORB orb) {
	
//			Get refrence to the GPS module.
		String url =
			"corbaloc::127.0.0.1:19740/ORB/OA/IDL:gps/GpsModule:1.0";
			
		org.omg.CORBA.Object ref = orb.string_to_object(url);
		
//			get the GPS object
		gpsModule = GpsModuleHelper.narrow(ref);
		
	}

	/** Method for reading the GPS position.
	 * This method must be called at least once before calling
	 * the different get-methods.
	 * 
	 * @throws NoGpsDataAvailableException
	 */
	public void read() throws NoGpsDataAvailableException {
		data = gpsModule.getGpsData();
		
		latdeg = data.position.lat.degrees;
		latmin = data.position.lat.minutes;
		latsec = data.position.lat.seconds;
		latmil = data.position.lat.milliseconds;
		
		lngdeg = data.position.lon.degrees;
		lngmin = data.position.lon.minutes;
		lngsec = data.position.lon.seconds;
		lngmil = data.position.lon.milliseconds;
		
	}
	
	public String getGpsData() {

		StringBuffer xml = new StringBuffer();
		
//		String gpsInformation = "");
//		String latitude = "";
//		StringBuffer longitude = new StringBuffer();
		
		try {
			
			xml.append("<gps>\n"); 
			xml.append("<seacageid>"+ Config.seacageid +"</seacageid>\n");
			
			read();
			
//			Handle GPS data.
			xml.append("<satellites>" + data.visibleSatellites + "</satellites>\n");
			xml.append("<altitude>" + data.alt + "</altitude>\n");
						
/*			
 			latitude = data.position.lat.degrees + " degrees ");
			latitude.append(data.position.lat.minutes + " minutes ");
			latitude.append(data.position.lat.seconds + " seconds ");
			latitude.append(data.position.lat.milliseconds + " milliseconds ");
			latitude.append(data.position.lat.type == NORTH.value ? "North" : "South");

			 
			xml.append("<latitude>\n");
			xml.append("<degrees>" + data.position.lat.degrees + "</degrees>\n");
			xml.append("<minutes>" + data.position.lat.minutes + "</minutes>\n");
			xml.append("<seconds>" + data.position.lat.seconds + "</seconds>\n");
			xml.append("<milliseconds>" + data.position.lat.milliseconds + "</milliseconds>\n");
			xml.append("<type>" + (data.position.lat.type == NORTH.value ? "North" : "South") + "</type>\n");
			xml.append("</latitude>\n");
			
			longitude.append(data.position.lon.degrees + " degrees ");
			longitude.append(data.position.lon.minutes + " minutes ");
			longitude.append(data.position.lon.seconds + " seconds ");
			longitude.append(data.position.lon.milliseconds + " milliseconds ");
			longitude.append(data.position.lon.type == EAST.value ? "East" : "West");
			

			xml.append("<longitude>\n");
			xml.append("<degrees>" + data.position.lon.degrees + "</degrees>\n");
			xml.append("<minutes>" + data.position.lon.minutes + "</minutes>\n");
			xml.append("<seconds>" + data.position.lon.seconds + "</seconds>\n");
			xml.append("<milliseconds>" + data.position.lon.milliseconds + "</milliseconds>\n");
			xml.append("<type>" + (data.position.lon.type == EAST.value ? "East" : "West") + "</type>\n");
			xml.append("</longitude>\n");
*/
			String lat = String.valueOf((latdeg + " " + latmin + " " + latsec));
			String lng = String.valueOf((lngdeg + " " + lngmin + " " + lngsec));

			xml.append("<latlng ");
			xml.append("lat=\"" + lat + "\" ");
			xml.append("lng=\"" + lng + "\" />");
			
		} catch (NoGpsDataAvailableException noGpsData) {
//			This exception is raised if the GPS device is disconnected
//			or if it has no satellite fix..
//			xml.append("<error class=\"GpsControl\" value=\"" + noGpsData.toString() + "\" />");
		}
		
		xml.append("</gps>\n");
//		log(xml.toString());
		return xml.toString();
			
	}
	
	/**
	 * @return the latitude degrees
	 */
	public long getLatdeg() {
		return latdeg;
	}

	/**
	 * @return the latitude minutes
	 */
	public long getLatmin() {
		return latmin;
	}

	/**
	 * @return the latitude seconds
	 */
	public long getLatsec() {
		return latsec;
	}

	/**
	 * @return the latitude milliseconds
	 */
	public long getLatmil() {
		return latmil;
	}

	/**
	 * @return the longitude degrees
	 */
	public long getLngdeg() {
		return lngdeg;
	}

	/**
	 * @return the longitude minutes
	 */
	public long getLngmin() {
		return lngmin;
	}

	/**
	 * @return the longitude seconds
	 */
	public long getLngsec() {
		return lngsec;
	}

	/**
	 * @return the longitude milliseconds
	 */
	public long getLngmil() {
		return lngmil;
	}
}
