package com.telenor.apms.fifamos.client.j2me.control;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;

import com.telenor.apms.fifamos.client.j2me.gui.ChoosePeriod;
import com.telenor.apms.fifamos.client.j2me.gui.FrameList;
import com.telenor.apms.fifamos.client.j2me.gui.SeaCageList;
import com.telenor.apms.fifamos.client.j2me.gui.SensorGraph;
import com.telenor.apms.fifamos.client.j2me.gui.SensorList;
import com.telenor.apms.fifamos.client.j2me.gui.Settings;
import com.telenor.apms.fifamos.client.j2me.gui.StartMenu;
import com.telenor.apms.fifamos.client.j2me.objects.Frame;
import com.telenor.apms.fifamos.client.j2me.objects.SeaCage;
import com.telenor.apms.fifamos.client.j2me.objects.Sensor;



/**
 * @author JAG
 * @copyright JAG  2006
 * @version 1.0
 */

public class MobileClient extends MIDlet implements CommandListener {
	
	private Display display;
	private FrameList fl;
	private SeaCageList scl;
	private SensorList sl;
	private ChoosePeriod cp;
	private Settings s;
	private StartMenu sm;
	private RecordStore rs;
	private Sensor currentSensor;
	private Frame currentFrame;
	private SeaCage currentSeaCage;
	
	public MobileClient() {
		// Obtain a reference to the device's UI
		display = Display.getDisplay(this);
		getSettings();
		sm = new StartMenu("StartMeny", List.IMPLICIT);
		sm.setCommandListener(this);
		display.setCurrent(sm);
	}

	public void commandAction(Command com, Displayable d) {
		//Back functionality
		if(com.getCommandType() == Command.BACK) {
			if (d instanceof FrameList) {
				sm.setCommandListener(this);
				display.setCurrent(sm);
			}
			if (d instanceof SeaCageList) {
				fl.setCommandListener(this);
				display.setCurrent(fl);
			}
			else if (d instanceof SensorList) {
				sl.stopTimer();
				scl.setCommandListener(this);
				display.setCurrent(scl);
			}
			else if (d instanceof ChoosePeriod) {
				sl.startTimer();
				sl.setCommandListener(this);
				display.setCurrent(sl);
			}
			else if (d instanceof SensorGraph) {
				cp.setCommandListener(this);
				display.setCurrent(cp);
			}
			
		//exit functionality	
		} else if (com.getCommandType() == Command.EXIT) {
			destroyApp(true);
			notifyDestroyed();
		
		//Browsing 	
		}  else if (d instanceof StartMenu && com == List.SELECT_COMMAND) {
			switch (((StartMenu)d).getSelectedIndex()) {
			case 0:
				fl = new FrameList("Velg frame", List.IMPLICIT);
				fl.setCommandListener(this);
				display.setCurrent(fl);
				break;
				
			case 1:
				s = new Settings("Innstillinger");
				s.setCommandListener(this);
				display.setCurrent(s);
				break;
			case 2:
				destroyApp(true);
				notifyDestroyed();
				break;
			}		
		}  else if (d instanceof FrameList && com == List.SELECT_COMMAND) {
			currentFrame = fl.getFrame(fl.getSelectedIndex());
			scl = new SeaCageList("Sea-cage liste", List.IMPLICIT, currentFrame.getID());
			scl.setCommandListener(this);
			display.setCurrent(scl);
		}  else if (d instanceof SeaCageList && com == List.SELECT_COMMAND) {
			currentSeaCage = scl.getSeaCage(scl.getSelectedIndex());
			sl = new SensorList("Sensordata: ", List.IMPLICIT, currentSeaCage.getID());
			sl.setCommandListener(this);
			display.setCurrent(sl);
		}  else if (d instanceof SensorList && com == List.SELECT_COMMAND) {
			sl.stopTimer();
			currentSensor = sl.getSensor(sl.getSelectedIndex());
			cp = new ChoosePeriod("Velg tidsperiode", List.IMPLICIT);
			cp.setCommandListener(this);
			display.setCurrent(cp);
		}  else if (d instanceof ChoosePeriod && com == List.SELECT_COMMAND) {
			long period =0;
			switch (cp.getSelectedIndex()) {
			case 0:
				period = Constants.PERIOD_HOUR;
				break;
			case 1:
				period = Constants.PERIOD_DAY;
				break;
			case 2:
				period = Constants.PERIOD_WEEK;
				break;
			case 3:
				period = Constants.PERIOD_MONTH;
				break;
			case 4:
				period = Constants.PERIOD_MINUTE;
				break;
			}
			SensorGraph sg = new SensorGraph(currentSensor, period);
			sg.setCommandListener(this);
			display.setCurrent(sg);
		}  
		//Save/load settings:
		else if (com.getCommandType() == Command.OK) {
			s.saveSettings();
			sm.setCommandListener(this);
			display.setCurrent(sm);
		}
		else if (com.getCommandType() == Command.CANCEL) {
			sm.setCommandListener(this);
			display.setCurrent(sm);
		}
	}// end commandAction( Command, Displayable )

	private void getSettings() {
		try {
			rs = RecordStore.openRecordStore("settings", false);
			Constants.SERVER_URL = new String(rs.getRecord(1));	
			Constants.INTERVAL = Integer.parseInt(new String(rs.getRecord(2)));
			rs.closeRecordStore();
		} catch(RecordStoreNotFoundException e) {
			e.printStackTrace();
		} catch (RecordStoreFullException e) {
			e.printStackTrace();
		} catch (RecordStoreException e) {
			e.printStackTrace();
		}
	}

	public void startApp() {
	}// end startApp()
	
	public void pauseApp() {
	}// end pauseApp()
	
	public void destroyApp(boolean unconditional) {
		display = null;
	}// end destoryApp( boolean )
}// end class PopMidlet
