package com.telenor.apms.fifamos.client.j2me.gui;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

import net.comtor.j2me.util.DateUtil;
import net.comtor.j2me.util.StringVector;

import org.kxmlrpc.TestXmlRpcClient;

import com.telenor.apms.fifamos.client.j2me.control.Constants;
import com.telenor.apms.fifamos.client.j2me.objects.Sensor;



/**
 * @author JAG
 * @copyright JAG  2006
 * @version 1.0
 */

public class SensorGraph extends Canvas implements Runnable{
	
	private Command exitCommand;
	private Command backCommand;
	int[][] toBeDrawn;
	long maxY = 2800;
	long minY;
	double maxX;
	double minX;
	int minAlarm;
	int maxAlarm;
	Sensor currentSensor;
	
	long period;
	long[][] sensordataTable;
	int height = (getHeight()-20);
	int width = (getWidth()-20);
	private double factorY;
	private long startTime;
	private long stopTime;
	
	public SensorGraph(Sensor currentSensor, long period) {
		exitCommand = new Command("Exit", Command.EXIT, 1);
		backCommand = new Command("Back", Command.BACK, 1);
		this.addCommand(exitCommand);
		this.addCommand(backCommand);
		this.period = period;
		this.currentSensor = currentSensor;
		minAlarm = (int)Double.parseDouble(currentSensor.getMinValue());
		maxAlarm = (int)Double.parseDouble(currentSensor.getMaxValue());
		new Thread(this).run();
	}
	
	public void run() {
		try {
			Vector params = new Vector();
			debug("SensorGraph contacting web service..");
			TestXmlRpcClient xmlrpc = new TestXmlRpcClient(
					Constants.SERVER_URL);
			params.addElement(currentSensor.getID());
			params.addElement(String.valueOf(period));
			
			
			String sensordata = (String) xmlrpc.execute(
					"Output.getHistoryForSensor", params);
			debug("sensordata: "+sensordata);
			StringVector sv = new StringVector(sensordata, ';');
			StringVector sv2 = new StringVector(sv.stringElementAt(0),':');
			sensordataTable = new long[sv.size()-1][2];
			
			minX=Long.parseLong(sv2.stringElementAt(0));//henter ut periodens frste timestamp
			maxX = Long.parseLong(sv2.stringElementAt(1));//henter ut periodens siste timestamp
			
			maxX=new Date().getTime()/1000;
			minX=(maxX-period);
			
			for(int i = 1; i<sv.size(); i++) {//frste element hoppes over. Her ligger kun timestamp som angir start og slutt av periode
				sv2 = new StringVector(sv.stringElementAt(i),':');

				for (int j = 0; j < sv2.size(); j++) {
					sensordataTable[i-1][j] = (long)Double.parseDouble(sv2.stringElementAt(j));
					//debug("sdt "+sensordataTable[i][j]);
				}
			}
			
		} catch (Exception e) {
			debug("ex "+e.toString());
		}
		toBeDrawn = normalize(sensordataTable);		
	}

	protected void paint(Graphics g) {
		
		//erase current display
		g.setColor(255,255,255);
		g.fillRect(0,0,getWidth(),getHeight());
		g.setColor(0,0,0);
		//Draw axis (x1,y1,x2,y2)
		g.drawLine(20,height,width,height);
		g.drawLine(20,height,20,20);
		
		Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
		g.setFont(f);
		
		g.drawString("Siste verdi av "+currentSensor.getTypeName()+": "+currentSensor.getCurrentValue()+" "+currentSensor.getUnit(), 25, 1, 0); //heading
		
		g.drawString(String.valueOf(maxY), 0, 15, 0);
		g.drawString(" 0", 0, (height-5), 0);

		Date start = new Date((long)minX*1000+7200000);
		Date end = new Date((long)maxX*1000+7200000);
		
			
		if(period==Constants.PERIOD_HOUR|period==Constants.PERIOD_MINUTE) {
			Date stopTimeDate = new Date((new Date().getTime()-(period*1000))+7200000);
			
			DateUtil du = new DateUtil();
			TimeZone tz = TimeZone.getTimeZone("GMT+2");
			Calendar cal = Calendar.getInstance(tz);
		    
			String startString = du.dateToStringOnlyHour(cal);
		    cal.setTime(stopTimeDate);
		    String endString = du.dateToStringOnlyHour(cal);

		    System.out.println(endString);
			System.out.println(startString);

			g.drawString(startString, 15 ,(height+2), 0);
			g.drawString(endString, (width) ,(height+2), Graphics.TOP|Graphics.RIGHT);
		}
		else {
			String endString = end.toString();
			String startString = start.toString();
			endString = endString.substring(0,10);
			startString = startString.substring(0,10);
			
			g.drawString(startString, 15 ,(height+2), 0);
			g.drawString(endString, (width) ,(height+2), Graphics.TOP|Graphics.RIGHT);
		}
		g.setColor(0,0,255);
		
		for(int i =0;i<(toBeDrawn.length-1);i++) {
			g.drawLine(       toBeDrawn[i][1], turn(toBeDrawn[i][0]), toBeDrawn[i+1][1], turn(toBeDrawn[i+1][0]));
		}

		//Draw alarm limits
		g.setColor(255,0,0);
		g.drawLine(20,turn((int)(maxAlarm*factorY)),width,turn((int)(maxAlarm*factorY)));
		g.drawLine(20,turn((int)(minAlarm*factorY)),width,turn((int)(minAlarm*factorY)));
	}

	private int[][] normalize(long [][] values) {//Normalize X and Y values to fit screen:
		debug("normalizing");
		int[][] res = new int[values.length][2];
				
		maxY = (long)Double.parseDouble(currentSensor.getMaxRange());
		
		//Find Y-value factor
		factorY = (height-20)/(double)maxY;
		debug("factor Y: "+factorY);
		
		//Find X-value factor
		double factorX = (width-20)/(maxX-minX);
		debug("factor X: "+factorX);
		
		//Store new Y- and X-values in result table
		for (int i = 0; i < values.length; i++) {
			values[i][1] = (values[i][1] - (long)minX);
			res[i][0] = (int)(values[i][0]*factorY);
			res[i][1] = (int)(values[i][1]*factorX+20);	
		}
		return res;
	}

	private int turn(int i) {
		return height-i;
	}

	private void debug(String string) {
		if(Constants.DEBUG) {
			System.out.println("SensorGraph DEBUG**** "+string);
		}
	}
}
