<%! 
	private static MBeanServer server;
	
	/**
	* Creates a object name
	*/
	public ObjectName createObjectName(String name, String type, String domain){
		
		try{
			return new ObjectName(domain+":Name="+name+", Type="+type);
		
		}catch(MalformedObjectNameException e){		
			System.out.println("Malformed object name: "+domain+":Name="+name+", Type="+type);
			e.printStackTrace(System.out);	
			System.exit(3);
		}
		
		return null;
	}
	
	/**
	* Gets all fact classes from a component package
	*/
	public Hashtable<String, Class> getFactClasses(String comID) throws IOException, SecurityException{
		try{
			ObjectName ifactory = createObjectName("InferenceFactory", "Core", "Module_Domain");
			return (Hashtable<String, Class>) invoke(ifactory, "getFactClasses", new Object[]{comID}, new String[]{String.class.getName()}, "Module_Domain");
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}
	

	/**
	* Gets a Engine (inference rules) object for a component
	*/
	public Engine getEngine(String comID) throws IOException, SecurityException{
		ObjectName ifactory = createObjectName("InferenceFactory", "Core", "Module_Domain");
		Engine engine = null;
		try{
			engine = (Engine) invoke(ifactory, "getEngine", new Object[]{comID}, new String[]{String.class.getName()}, "Module_Domain");
		}catch(MBeanException e){
			engine = null;
		}
		
		return engine;
	}

	/**
	* Gets a list of loaded components from CoMS
	*/
	public LinkedHashSet<ComponentInfo> getLoadedList() throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			return (LinkedHashSet<ComponentInfo>) getAttribute(cm, "ComponentList", "Module_Domain");
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	* Gets a list of loaded packages from CoMS
	*/
	public LinkedHashMap<String, ArrayList<String>> getPackageList() throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			return (LinkedHashMap<String, ArrayList<String>>) getAttribute(cm, "PackageList", "Module_Domain");
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}

	/**
	* Removes a package from the CoMS
	*/
	public Boolean removePackage(String pkg) throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			Boolean success =  (Boolean) invoke(cm, "removePackage", new Object[]{pkg}, 
														new String[]{String.class.getName()}, "Module_Domain");
			return success;
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}

	/**
	* Gets the config options for a component
	*/
	public HashMap<String, String> getComponentConfig(String comID) throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			return (HashMap<String, String>) invoke(cm, "_getComponentConfig", new Object[]{comID}, 
															new String[]{String.class.getName()}, "Module_Domain");
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	* Change the config options for a component
	*/
	public Boolean changeComponentConfig(String comID, HashMap configMap) throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			Boolean success = (Boolean) invoke(cm, "changeComponentConfig", new Object[]{comID, configMap}, 
														new String[]{String.class.getName(), HashMap.class.getName()}, "Module_Domain");
			return success;
			
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
		
	}
	
	
	/**
	* Stops a component
	*/
	public LinkedHashMap<String, String> stopComponent(String comID) throws IOException, SecurityException{
		try{
		ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
		return (LinkedHashMap<String, String>) invoke(cm, "stopComponent", new Object[]{comID}, 
													new String[]{String.class.getName()}, "Module_Domain");
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	* Starts a component
	*/
	public LinkedHashMap<String, String> startComponent(String comID) throws IOException, SecurityException{
		try{
			ObjectName cm = createObjectName("ComponentManager", "Core", "Module_Domain");
			return (LinkedHashMap<String, String>) invoke(cm, "startComponent", new Object[]{comID}, 
														new String[]{String.class.getName()}, "Module_Domain");
		}catch(MBeanException e){
			e.printStackTrace();
			return null;
		}
	}
	
	/**
	* Gets an attribute from a mbean
	* @param ObjectName describing the module, String describing which attribute, String describing the domain
	* @return An Object with the attribute
	*/
	private Object getAttribute(ObjectName module, String attr, String domain) throws MBeanException{
		try{
			getServerHandle(domain);
			Object result = server.getAttribute(module, attr);
			return result;
		}catch(AttributeNotFoundException e){
			log("getAttribute(): Attribute \""+attr+"\" was not found\nModule: "+ module.toString());
		}catch(InstanceNotFoundException e){
			log("getAttribute(): Instance not found exception\nModule: "+ module.toString());
		}catch(ReflectionException e){
    		log("getAttribute(): Reflection exception \nModule: "+ module.toString());
    	}
		
		return null;
	}

	/**
	* Invokes a function on an mbean
	*/
	public Object invoke(ObjectName module, String method, Object[] args, String[] types, String domain) throws MBeanException{
		try{
			
			getServerHandle(domain);
			return server.invoke(module, method, args, types);
		
		}catch(InstanceNotFoundException e){
    		e.printStackTrace();
    		System.exit(0);
    	}catch(ReflectionException e){
    		e.printStackTrace();
    		System.exit(0);
    	}
    	
    		return null;
	}
	
	/**
	* Gets the server handle for the domain specified
	* @param String describing the domain
	*/
	private void getServerHandle(String domain){
    		
			ArrayList mbeanservers = MBeanServerFactory.findMBeanServer(null);
			Iterator iter = mbeanservers.iterator();
		
			while(iter.hasNext()){
				server = (MBeanServer)iter.next();
				
			if(server.getDefaultDomain().equals(domain))
				break;
			}
		
			if(server == null){
				System.out.println("Could not find mbeanserver handle");
				System.exit(0);
			}
	}
		
%>